@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "api/api_common.h"
 
+#include "base/qt/qt_key_modifiers.h"
 #include "data/data_thread.h"
 
 namespace Api {
@@ -20,4 +21,11 @@ SendAction::SendAction(
 , topicRootId(replyTo) {
 }
 
+SendOptions DefaultSendWhenOnlineOptions() {
+	return {
+		.scheduled = kScheduledUntilOnlineTimestamp,
+		.silent = base::IsCtrlPressed(),
+	};
+}
+
 } // namespace Api
