@@ -34,8 +34,18 @@ enum class WhoReactedList {
 	not_null<HistoryItem*> item,
 	WhoReactedList list);
 
+struct WhoReadPeer {
+	PeerId peer = 0;
+	TimeId date = 0;
+	bool dateReacted = false;
+
+	friend inline bool operator==(
+		const WhoReadPeer &a,
+		const WhoReadPeer &b) noexcept = default;
+};
+
 struct WhoReadList {
-	std::vector<PeerId> list;
+	std::vector<WhoReadPeer> list;
 	Ui::WhoReadType type = {};
 };
 
