@@ -10,23 +10,30 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lang/lang_keys.h"
 #include "ui/effects/round_checkbox.h"
 #include "ui/image/image.h"
+#include "ui/chat/attach/attach_extensions.h"
 #include "ui/chat/chat_theme.h"
 #include "ui/ui_utility.h"
 #include "main/main_session.h"
 #include "apiwrap.h"
 #include "mtproto/sender.h"
+#include "core/file_utilities.h"
+#include "data/data_peer.h"
 #include "data/data_session.h"
 #include "data/data_file_origin.h"
 #include "data/data_document.h"
 #include "data/data_document_media.h"
 #include "boxes/background_preview_box.h"
+#include "info/profile/info_profile_icon.h"
+#include "settings/settings_common.h"
 #include "ui/boxes/confirm_box.h"
+#include "ui/widgets/buttons.h"
 #include "window/window_session_controller.h"
 #include "window/themes/window_theme.h"
 #include "styles/style_overview.h"
 #include "styles/style_layers.h"
 #include "styles/style_boxes.h"
 #include "styles/style_chat_helpers.h"
+#include "styles/style_info.h"
 
 namespace {
 
@@ -61,11 +68,15 @@ class BackgroundBox::Inner final : public Ui::RpWidget {
 public:
 	Inner(
 		QWidget *parent,
-		not_null<Main::Session*> session);
+		not_null<Main::Session*> session,
+		PeerData *forPeer);
 	~Inner();
 
-	rpl::producer<Data::WallPaper> chooseEvents() const;
-	rpl::producer<Data::WallPaper> removeRequests() const;
+	[[nodiscard]] rpl::producer<Data::WallPaper> chooseEvents() const;
+	[[nodiscard]] rpl::producer<Data::WallPaper> removeRequests() const;
+
+	[[nodiscard]] auto resolveResetCustomPaper() const
+		->std::optional<Data::WallPaper>;
 
 	void removePaper(const Data::WallPaper &data);
 
@@ -109,6 +120,7 @@ private:
 	void resizeToContentAndPreload();
 	void updatePapers();
 	void requestPapers();
+	void pushCustomPapers();
 	void sortPapers();
 	void paintPaper(
 		QPainter &p,
@@ -118,9 +130,13 @@ private:
 	void validatePaperThumbnail(const Paper &paper) const;
 
 	const not_null<Main::Session*> _session;
+	PeerData * const _forPeer = nullptr;
+
 	MTP::Sender _api;
 
 	std::vector<Paper> _papers;
+	uint64 _currentId = 0;
+	uint64 _insertedResetId = 0;
 
 	Selection _over;
 	Selection _overDown;
@@ -133,8 +149,10 @@ private:
 
 BackgroundBox::BackgroundBox(
 	QWidget*,
-	not_null<Window::SessionController*> controller)
-: _controller(controller) {
+	not_null<Window::SessionController*> controller,
+	PeerData *forPeer)
+: _controller(controller)
+, _forPeer(forPeer) {
 }
 
 void BackgroundBox::prepare() {
@@ -144,15 +162,38 @@ void BackgroundBox::prepare() {
 
 	setDimensions(st::boxWideWidth, st::boxMaxListHeight);
 
-	_inner = setInnerWidget(
-		object_ptr<Inner>(this, &_controller->session()),
-		st::backgroundScroll);
+	auto wrap = object_ptr<Ui::VerticalLayout>(this);
+	const auto container = wrap.data();
+
+	Settings::AddSkip(container);
+
+	const auto button = container->add(Settings::CreateButton(
+		container,
+		tr::lng_settings_bg_from_file(),
+		st::infoProfileButton));
+	object_ptr<Info::Profile::FloatingIcon>(
+		button,
+		st::infoIconMediaPhoto,
+		st::infoSharedMediaButtonIconPosition);
+
+	button->setClickedCallback([=] {
+		chooseFromFile();
+	});
+
+	Settings::AddSkip(container);
+	Settings::AddDivider(container);
+
+	_inner = container->add(
+		object_ptr<Inner>(this, &_controller->session(), _forPeer));
+
+	container->resizeToWidth(st::boxWideWidth);
+
+	setInnerWidget(std::move(wrap), st::backgroundScroll);
+	setInnerTopSkip(st::lineWidth);
 
 	_inner->chooseEvents(
 	) | rpl::start_with_next([=](const Data::WallPaper &paper) {
-		_controller->show(
-			Box<BackgroundPreviewBox>(_controller, paper),
-			Ui::LayerOption::KeepOther);
+		chosen(paper);
 	}, _inner->lifetime());
 
 	_inner->removeRequests(
@@ -161,6 +202,122 @@ void BackgroundBox::prepare() {
 	}, _inner->lifetime());
 }
 
+void BackgroundBox::chooseFromFile() {
+	const auto filterStart = _forPeer
+		? u"Image files (*"_q
+		: u"Theme files (*.tdesktop-theme *.tdesktop-palette *"_q;
+	auto filters = QStringList(
+		filterStart
+		+ Ui::ImageExtensions().join(u" *"_q)
+		+ u")"_q);
+	filters.push_back(FileDialog::AllFilesFilter());
+	const auto callback = [=](const FileDialog::OpenResult &result) {
+		if (result.paths.isEmpty() && result.remoteContent.isEmpty()) {
+			return;
+		}
+
+		if (!_forPeer && !result.paths.isEmpty()) {
+			const auto filePath = result.paths.front();
+			const auto hasExtension = [&](QLatin1String extension) {
+				return filePath.endsWith(extension, Qt::CaseInsensitive);
+			};
+			if (hasExtension(qstr(".tdesktop-theme"))
+				|| hasExtension(qstr(".tdesktop-palette"))) {
+				Window::Theme::Apply(filePath);
+				return;
+			}
+		}
+
+		auto image = Images::Read({
+			.path = result.paths.isEmpty() ? QString() : result.paths.front(),
+			.content = result.remoteContent,
+			.forceOpaque = true,
+		}).image;
+		if (image.isNull() || image.width() <= 0 || image.height() <= 0) {
+			return;
+		}
+		auto local = Data::CustomWallPaper();
+		local.setLocalImageAsThumbnail(std::make_shared<Image>(
+			std::move(image)));
+		_controller->show(Box<BackgroundPreviewBox>(
+			_controller,
+			local,
+			BackgroundPreviewArgs{ _forPeer }));
+	};
+	FileDialog::GetOpenPath(
+		this,
+		tr::lng_choose_image(tr::now),
+		filters.join(u";;"_q),
+		crl::guard(this, callback));
+}
+
+bool BackgroundBox::hasDefaultForPeer() const {
+	Expects(_forPeer != nullptr);
+
+	const auto paper = _forPeer->wallPaper();
+	if (!paper) {
+		return true;
+	}
+	const auto reset = _inner->resolveResetCustomPaper();
+	Assert(reset.has_value());
+	return (paper->id() == reset->id());
+}
+
+bool BackgroundBox::chosenDefaultForPeer(
+		const Data::WallPaper &paper) const {
+	if (!_forPeer) {
+		return false;
+	}
+
+	const auto reset = _inner->resolveResetCustomPaper();
+	Assert(reset.has_value());
+	return (paper.id() == reset->id());
+}
+
+void BackgroundBox::chosen(const Data::WallPaper &paper) {
+	if (chosenDefaultForPeer(paper)) {
+		if (!hasDefaultForPeer()) {
+			const auto reset = crl::guard(this, [=](Fn<void()> close) {
+				resetForPeer();
+				close();
+			});
+			_controller->show(Ui::MakeConfirmBox({
+				.text = u"Are you sure you want to reset the wallpaper?"_q,
+				.confirmed = reset,
+				.confirmText = u"Reset"_q,
+			}));
+		} else {
+			closeBox();
+		}
+		return;
+	}
+	_controller->show(
+		Box<BackgroundPreviewBox>(
+			_controller,
+			paper,
+			BackgroundPreviewArgs{ _forPeer }),
+		Ui::LayerOption::KeepOther);
+}
+
+void BackgroundBox::resetForPeer() {
+	const auto api = &_controller->session().api();
+	api->request(MTPmessages_SetChatWallPaper(
+		MTP_flags(0),
+		_forPeer->input,
+		MTPInputWallPaper(),
+		MTPWallPaperSettings(),
+		MTPint()
+	)).done([=](const MTPUpdates &result) {
+		api->applyUpdates(result);
+	}).send();
+
+	const auto weak = Ui::MakeWeak(this);
+	_forPeer->setWallPaper(std::nullopt);
+	if (weak) {
+		_controller->finishChatThemeEdit(_forPeer);
+	}
+}
+
 void BackgroundBox::removePaper(const Data::WallPaper &paper) {
 	const auto session = &_controller->session();
 	const auto remove = [=, weak = Ui::MakeWeak(this)](Fn<void()> &&close) {
@@ -186,17 +343,19 @@ void BackgroundBox::removePaper(const Data::WallPaper &paper) {
 
 BackgroundBox::Inner::Inner(
 	QWidget *parent,
-	not_null<Main::Session*> session)
+	not_null<Main::Session*> session,
+	PeerData *forPeer)
 : RpWidget(parent)
 , _session(session)
+, _forPeer(forPeer)
 , _api(&_session->mtp())
 , _check(std::make_unique<Ui::RoundCheckbox>(st::overviewCheck, [=] { update(); })) {
 	_check->setChecked(true, anim::type::instant);
-	if (_session->data().wallpapers().empty()) {
-		resize(st::boxWideWidth, 2 * (st::backgroundSize.height() + st::backgroundPadding) + st::backgroundPadding);
-	} else {
+	resize(st::boxWideWidth, 2 * (st::backgroundSize.height() + st::backgroundPadding) + st::backgroundPadding);
+	Window::Theme::IsNightModeValue(
+	) | rpl::start_with_next([=] {
 		updatePapers();
-	}
+	}, lifetime());
 	requestPapers();
 
 	_session->downloaderTaskFinished(
@@ -219,6 +378,7 @@ BackgroundBox::Inner::Inner(
 		}
 	}, lifetime());
 
+
 	setMouseTracking(true);
 }
 
@@ -232,35 +392,106 @@ void BackgroundBox::Inner::requestPapers() {
 	}).send();
 }
 
+auto BackgroundBox::Inner::resolveResetCustomPaper() const
+-> std::optional<Data::WallPaper> {
+	if (!_forPeer) {
+		return {};
+	}
+	const auto nonCustom = Window::Theme::Background()->paper();
+	const auto themeEmoji = _forPeer->themeEmoji();
+	if (themeEmoji.isEmpty()) {
+		return nonCustom;
+	}
+	const auto &themes = _forPeer->owner().cloudThemes();
+	const auto theme = themes.themeForEmoji(themeEmoji);
+	if (!theme) {
+		return nonCustom;
+	}
+	using Type = Data::CloudTheme::Type;
+	const auto dark = Window::Theme::IsNightMode();
+	const auto i = theme->settings.find(dark ? Type::Dark : Type::Light);
+	if (i != end(theme->settings) && i->second.paper) {
+		return *i->second.paper;
+	}
+	return nonCustom;
+}
+
+void BackgroundBox::Inner::pushCustomPapers() {
+	auto customId = uint64();
+	if (const auto custom = _forPeer ? _forPeer->wallPaper() : nullptr) {
+		customId = custom->id();
+		const auto j = ranges::find(
+			_papers,
+			custom->id(),
+			[](const Paper &paper) { return paper.data.id(); });
+		if (j != end(_papers)) {
+			j->data = j->data.withParamsFrom(*custom);
+		} else {
+			_papers.insert(begin(_papers), Paper{ *custom });
+		}
+	}
+	if (const auto reset = resolveResetCustomPaper()) {
+		_insertedResetId = reset->id();
+		const auto j = ranges::find(
+			_papers,
+			_insertedResetId,
+			[](const Paper &paper) { return paper.data.id(); });
+		if (j != end(_papers)) {
+			if (_insertedResetId != customId) {
+				j->data = j->data.withParamsFrom(*reset);
+			}
+		} else {
+			_papers.insert(begin(_papers), Paper{ *reset });
+		}
+	}
+}
+
 void BackgroundBox::Inner::sortPapers() {
-	const auto current = Window::Theme::Background()->id();
-	const auto night = Window::Theme::IsNightMode();
+	const auto currentCustom = _forPeer ? _forPeer->wallPaper() : nullptr;
+	_currentId = currentCustom
+		? currentCustom->id()
+		: _insertedResetId
+		? _insertedResetId
+		: Window::Theme::Background()->id();
+	const auto dark = Window::Theme::IsNightMode();
 	ranges::stable_sort(_papers, std::greater<>(), [&](const Paper &paper) {
 		const auto &data = paper.data;
 		return std::make_tuple(
-			data.id() == current,
-			night ? data.isDark() : !data.isDark(),
+			_insertedResetId && (_insertedResetId == data.id()),
+			data.id() == _currentId,
+			dark ? data.isDark() : !data.isDark(),
 			Data::IsDefaultWallPaper(data),
 			!data.isDefault() && !Data::IsLegacy1DefaultWallPaper(data),
 			Data::IsLegacy3DefaultWallPaper(data),
 			Data::IsLegacy2DefaultWallPaper(data),
 			Data::IsLegacy1DefaultWallPaper(data));
 	});
-	if (!_papers.empty() && _papers.front().data.id() == current) {
+	if (!_papers.empty()
+		&& _papers.front().data.id() == _currentId
+		&& !currentCustom
+		&& !_insertedResetId) {
 		_papers.front().data = _papers.front().data.withParamsFrom(
 			Window::Theme::Background()->paper());
 	}
 }
 
 void BackgroundBox::Inner::updatePapers() {
+	if (_session->data().wallpapers().empty()) {
+		return;
+	}
 	_over = _overDown = Selection();
 
 	_papers = _session->data().wallpapers(
-	) | ranges::views::filter([](const Data::WallPaper &paper) {
-		return !paper.isPattern() || !paper.backgroundColors().empty();
+	) | ranges::views::filter([&](const Data::WallPaper &paper) {
+		return (!paper.isPattern() || !paper.backgroundColors().empty())
+			&& (!_forPeer
+				|| (!Data::IsDefaultWallPaper(paper)
+					&& (Data::IsCloudWallPaper(paper)
+						|| Data::IsCustomWallPaper(paper))));
 	}) | ranges::views::transform([](const Data::WallPaper &paper) {
 		return Paper{ paper };
 	}) | ranges::to_vector;
+	pushCustomPapers();
 	sortPapers();
 	resizeToContentAndPreload();
 }
@@ -373,7 +604,7 @@ void BackgroundBox::Inner::paintPaper(
 	}
 
 	const auto over = !v::is_null(_overDown) ? _overDown : _over;
-	if (paper.data.id() == Window::Theme::Background()->id()) {
+	if (paper.data.id() == _currentId) {
 		const auto checkLeft = x + st::backgroundSize.width() - st::overviewCheckSkip - st::overviewCheck.size;
 		const auto checkTop = y + st::backgroundSize.height() - st::overviewCheckSkip - st::overviewCheck.size;
 		_check->paint(p, checkLeft, checkTop, width());
@@ -415,14 +646,13 @@ void BackgroundBox::Inner::mouseMoveEvent(QMouseEvent *e) {
 			- st::stickerPanDeleteIconBg.width();
 		const auto deleteBottom = row * (height + skip) + skip
 			+ st::stickerPanDeleteIconBg.height();
-		const auto currentId = Window::Theme::Background()->id();
 		const auto inDelete = (x >= deleteLeft)
 			&& (y < deleteBottom)
 			&& Data::IsCloudWallPaper(data)
 			&& !Data::IsDefaultWallPaper(data)
 			&& !Data::IsLegacy2DefaultWallPaper(data)
 			&& !Data::IsLegacy3DefaultWallPaper(data)
-			&& (currentId != data.id());
+			&& (_currentId != data.id());
 		return (result >= _papers.size())
 			? Selection()
 			: inDelete
