@@ -9,6 +9,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "boxes/abstract_box.h"
 
+class PeerData;
+
 namespace Window {
 class SessionController;
 } // namespace Window
@@ -19,7 +21,10 @@ class WallPaper;
 
 class BackgroundBox : public Ui::BoxContent {
 public:
-	BackgroundBox(QWidget*, not_null<Window::SessionController*> controller);
+	BackgroundBox(
+		QWidget*,
+		not_null<Window::SessionController*> controller,
+		PeerData *forPeer = nullptr);
 
 protected:
 	void prepare() override;
@@ -27,10 +32,18 @@ protected:
 private:
 	class Inner;
 
+	void chosen(const Data::WallPaper &paper);
+	[[nodiscard]] bool hasDefaultForPeer() const;
+	[[nodiscard]] bool chosenDefaultForPeer(
+		const Data::WallPaper &paper) const;
 	void removePaper(const Data::WallPaper &paper);
+	void resetForPeer();
+
+	void chooseFromFile();
 
 	const not_null<Window::SessionController*> _controller;
 
 	QPointer<Inner> _inner;
+	PeerData *_forPeer = nullptr;
 
 };
