@@ -8,20 +8,25 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/filters/edit_filter_box.h"
 
 #include "boxes/filters/edit_filter_chats_list.h"
+#include "boxes/filters/edit_filter_links.h"
+#include "boxes/premium_limits_box.h"
 #include "chat_helpers/emoji_suggestions_widget.h"
 #include "ui/layers/generic_box.h"
 #include "ui/text/text_utilities.h"
 #include "ui/text/text_options.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/input_fields.h"
+#include "ui/wrap/slide_wrap.h"
 #include "ui/effects/panel_animation.h"
 #include "ui/filter_icons.h"
 #include "ui/filter_icon_panel.h"
 #include "ui/painter.h"
+#include "data/data_channel.h"
 #include "data/data_chat_filters.h"
 #include "data/data_peer.h"
 #include "data/data_peer_values.h" // Data::AmPremiumValue.
 #include "data/data_session.h"
+#include "data/data_user.h"
 #include "core/application.h"
 #include "core/core_settings.h"
 #include "settings/settings_common.h"
@@ -37,6 +42,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_layers.h"
 #include "styles/style_window.h"
 #include "styles/style_chat.h"
+#include "styles/style_menu_icons.h"
 
 namespace {
 
@@ -499,6 +505,24 @@ void CreateIconSelector(
 	return QString();
 }
 
+not_null<Ui::SettingsButton*> AddToggledButton(
+		not_null<Ui::VerticalLayout*> container,
+		rpl::producer<bool> shown,
+		rpl::producer<QString> text,
+		const style::SettingsButton &st,
+		IconDescriptor &&descriptor) {
+	const auto toggled = container->add(
+		object_ptr<Ui::SlideWrap<Ui::SettingsButton>>(
+			container,
+			CreateButton(
+				container,
+				std::move(text),
+				st,
+				std::move(descriptor)))
+	)->toggleOn(std::move(shown), anim::type::instant)->setDuration(0);
+	return toggled->entity();
+}
+
 [[nodiscard]] QString TrimDefaultTitle(const QString &title) {
 	return (title.size() <= kMaxFilterTitleLength) ? title : QString();
 }
@@ -509,10 +533,58 @@ void EditFilterBox(
 		not_null<Ui::GenericBox*> box,
 		not_null<Window::SessionController*> window,
 		const Data::ChatFilter &filter,
-		Fn<void(const Data::ChatFilter &)> doneCallback) {
-	const auto creating = filter.title().isEmpty();
+		Fn<void(const Data::ChatFilter &)> doneCallback,
+		Fn<void(
+			const Data::ChatFilter &data,
+			Fn<void(Data::ChatFilter)> next)> saveAnd) {
+	using namespace rpl::mappers;
+
+	struct State {
+		rpl::variable<Data::ChatFilter> rules;
+		rpl::variable<std::vector<Data::ChatFilterLink>> links;
+		rpl::variable<bool> hasLinks;
+		rpl::variable<bool> chatlist;
+		rpl::variable<bool> creating;
+	};
+	const auto owner = &window->session().data();
+	const auto state = box->lifetime().make_state<State>(State{
+		.rules = filter,
+		.chatlist = filter.chatlist(),
+		.creating = filter.title().isEmpty(),
+	});
+	state->links = owner->chatsFilters().chatlistLinks(filter.id()),
+	state->hasLinks = state->links.value() | rpl::map([=](const auto &v) {
+		return !v.empty();
+	});
+	state->hasLinks.value() | rpl::filter(
+		_1
+	) | rpl::start_with_next([=] {
+		state->chatlist = true;
+	}, box->lifetime());
+
+	const auto data = &state->rules;
+
+	owner->chatsFilters().isChatlistChanged(
+	) | rpl::filter([=](FilterId id) {
+		return (id == data->current().id());
+	}) | rpl::start_with_next([=](FilterId id) {
+		const auto filters = &owner->chatsFilters();
+		const auto &list = filters->list();
+		const auto i = ranges::find(list, id, &Data::ChatFilter::id);
+		if (i == end(list)) {
+			return;
+		}
+		*data = data->current().withChatlist(i->chatlist(), i->hasMyLinks());
+		if (!i->chatlist() && !state->hasLinks.current()) {
+			state->chatlist = false;
+		}
+	}, box->lifetime());
+
 	box->setWidth(st::boxWideWidth);
-	box->setTitle(creating ? tr::lng_filters_new() : tr::lng_filters_edit());
+	box->setTitle(rpl::conditional(
+		state->creating.value(),
+		tr::lng_filters_new(),
+		tr::lng_filters_edit()));
 	box->setCloseByOutsideClick(false);
 
 	Data::AmPremiumValue(
@@ -521,9 +593,6 @@ void EditFilterBox(
 		box->closeBox();
 	}, box->lifetime());
 
-	using State = rpl::variable<Data::ChatFilter>;
-	const auto data = box->lifetime().make_state<State>(filter);
-
 	const auto content = box->verticalLayout();
 	const auto name = content->add(
 		object_ptr<Ui::InputField>(
@@ -543,7 +612,12 @@ void EditFilterBox(
 
 	const auto nameEditing = box->lifetime().make_state<NameEditing>(
 		NameEditing{ name });
-	nameEditing->custom = !creating;
+
+	state->creating.value(
+	) | rpl::filter(!_1) | rpl::start_with_next([=] {
+		nameEditing->custom = true;
+	}, box->lifetime());
+
 	QObject::connect(name, &Ui::InputField::changed, [=] {
 		if (!nameEditing->settingDefault) {
 			nameEditing->custom = true;
@@ -604,24 +678,141 @@ void EditFilterBox(
 	AddDividerText(content, tr::lng_filters_include_about());
 	AddSkip(content);
 
-	AddSubsectionTitle(content, tr::lng_filters_exclude());
+	auto excludeWrap = content->add(
+		object_ptr<Ui::SlideWrap<Ui::VerticalLayout>>(
+			content,
+			object_ptr<Ui::VerticalLayout>(content))
+	)->setDuration(0);
+	excludeWrap->toggleOn(state->chatlist.value() | rpl::map(!_1));
+	const auto excludeInner = excludeWrap->entity();
+
+	AddSubsectionTitle(excludeInner, tr::lng_filters_exclude());
 
 	const auto excludeAdd = AddButton(
-		content,
+		excludeInner,
 		tr::lng_filters_remove_chats(),
 		st::settingsButtonActive,
 		{ &st::settingsIconRemove, 0, IconType::Round, &st::windowBgActive });
 
 	const auto exclude = SetupChatsPreview(
-		content,
+		excludeInner,
 		data,
 		updateDefaultTitle,
 		kExcludeTypes,
 		&Data::ChatFilter::never);
 
-	AddSkip(content);
-	AddDividerText(content, tr::lng_filters_exclude_about());
+	AddSkip(excludeInner);
+	AddDividerText(excludeInner, tr::lng_filters_exclude_about());
+	AddSkip(excludeInner);
 
+	const auto collect = [=]() -> std::optional<Data::ChatFilter> {
+		const auto title = name->getLastText().trimmed();
+		const auto rules = data->current();
+		if (title.isEmpty()) {
+			name->showError();
+			box->scrollToY(0);
+			return {};
+		} else if (!(rules.flags() & kTypes) && rules.always().empty()) {
+			window->window().showToast(tr::lng_filters_empty(tr::now));
+			return {};
+		} else if ((rules.flags() == (kTypes | Flag::NoArchived))
+			&& rules.always().empty()
+			&& rules.never().empty()) {
+			window->window().showToast(tr::lng_filters_default(tr::now));
+			return {};
+		}
+		return rules.withTitle(title);
+	};
+
+	AddSubsectionTitle(
+		content,
+		rpl::conditional(
+			state->hasLinks.value(),
+			tr::lng_filters_link_has(),
+			tr::lng_filters_link()));
+
+	state->hasLinks.changes() | rpl::start_with_next([=] {
+		content->resizeToWidth(content->widthNoMargins());
+	}, content->lifetime());
+
+	if (filter.chatlist()) {
+		window->session().data().chatsFilters().reloadChatlistLinks(
+			filter.id());
+	}
+
+	const auto createLink = AddToggledButton(
+		content,
+		state->hasLinks.value() | rpl::map(!rpl::mappers::_1),
+		tr::lng_filters_link_create(),
+		st::settingsButtonActive,
+		{ &st::settingsFolderShareIcon, 0, IconType::Simple });
+	const auto addLink = AddToggledButton(
+		content,
+		state->hasLinks.value(),
+		tr::lng_group_invite_add(),
+		st::settingsButtonActive,
+		{ &st::settingsIconAdd, 0, IconType::Round, &st::windowBgActive });
+
+	SetupFilterLinks(
+		content,
+		window,
+		state->links.value(),
+		[=] { return collect().value_or(Data::ChatFilter()); });
+
+	rpl::merge(
+		createLink->clicks(),
+		addLink->clicks()
+	) | rpl::filter(
+		(rpl::mappers::_1 == Qt::LeftButton)
+	) | rpl::start_with_next([=](Qt::MouseButton button) {
+		const auto result = collect();
+		if (!result || !GoodForExportFilterLink(window, *result)) {
+			return;
+		}
+		const auto shared = CollectFilterLinkChats(*result);
+		if (shared.empty()) {
+			window->show(ShowLinkBox(window, *result, {}));
+			return;
+		}
+		saveAnd(*result, crl::guard(box, [=](Data::ChatFilter updated) {
+			state->creating = false;
+
+			// Comparison of ChatFilter-s don't take id into account!
+			data->force_assign(updated);
+			const auto id = updated.id();
+			state->links = owner->chatsFilters().chatlistLinks(id);
+			ExportFilterLink(id, shared, crl::guard(box, [=](
+					Data::ChatFilterLink link) {
+				Expects(link.id == id);
+
+				*data = data->current().withChatlist(true, true);
+				window->show(ShowLinkBox(window, updated, link));
+			}), crl::guard(box, [=](QString error) {
+				const auto session = &window->session();
+				if (error == u"CHATLISTS_TOO_MUCH"_q) {
+					window->show(Box(ShareableFiltersLimitBox, session));
+				} else if (error == u"INVITES_TOO_MUCH"_q) {
+					window->show(Box(FilterLinksLimitBox, session));
+				} else if (error == u"CHANNELS_TOO_MUCH"_q) {
+					window->show(Box(ChannelsLimitBox, session));
+				} else if (error == u"USER_CHANNELS_TOO_MUCH"_q) {
+					window->showToast(
+						{ tr::lng_filters_link_group_admin_error(tr::now) });
+				} else {
+					window->show(ShowLinkBox(window, updated, { .id = id }));
+				}
+			}));
+		}));
+	}, createLink->lifetime());
+	AddSkip(content);
+	AddDividerText(
+		content,
+		rpl::conditional(
+			state->hasLinks.value(),
+			tr::lng_filters_link_about_many(),
+			tr::lng_filters_link_about()));
+
+	const auto show = std::make_shared<Ui::BoxShow>(box);
 	const auto refreshPreviews = [=] {
 		include->updateData(
 			data->current().flags() & kTypes,
@@ -634,7 +825,7 @@ void EditFilterBox(
 		EditExceptions(
 			window,
 			box,
-			kTypes,
+			kTypes | (state->chatlist.current() ? Flag::Chatlist : Flag()),
 			data,
 			updateDefaultTitle,
 			refreshPreviews);
@@ -650,35 +841,17 @@ void EditFilterBox(
 	});
 
 	const auto save = [=] {
-		const auto title = name->getLastText().trimmed();
-		const auto rules = data->current();
-		const auto result = Data::ChatFilter(
-			rules.id(),
-			title,
-			rules.iconEmoji(),
-			rules.flags(),
-			rules.always(),
-			rules.pinned(),
-			rules.never());
-		if (title.isEmpty()) {
-			name->showError();
-			return;
-		} else if (!(rules.flags() & kTypes) && rules.always().empty()) {
-			window->window().showToast(tr::lng_filters_empty(tr::now));
-			return;
-		} else if ((rules.flags() == (kTypes | Flag::NoArchived))
-			&& rules.always().empty()
-			&& rules.never().empty()) {
-			window->window().showToast(tr::lng_filters_default(tr::now));
-			return;
+		if (const auto result = collect()) {
+			box->closeBox();
+			doneCallback(*result);
 		}
-		box->closeBox();
-
-		doneCallback(result);
 	};
-	box->addButton(
-		creating ? tr::lng_filters_create_button() : tr::lng_settings_save(),
-		save);
+
+	box->addButton(rpl::conditional(
+		state->creating.value(),
+		tr::lng_filters_create_button(),
+		tr::lng_settings_save()
+	), save);
 	box->addButton(tr::lng_cancel(), [=] { box->closeBox(); });
 }
 
@@ -707,9 +880,16 @@ void EditExistingFilter(
 			tl
 		)).send();
 	};
+	const auto saveAnd = [=](
+			const Data::ChatFilter &data,
+			Fn<void(Data::ChatFilter)> next) {
+		doneCallback(data);
+		next(data);
+	};
 	window->window().show(Box(
 		EditFilterBox,
 		window,
 		*i,
-		crl::guard(session, doneCallback)));
+		crl::guard(session, doneCallback),
+		crl::guard(session, saveAnd)));
 }
