@@ -343,9 +343,10 @@ EditFilterChatsListController::EditFilterChatsListController(
 , _session(session)
 , _title(std::move(title))
 , _peers(peers)
-, _options(options)
+, _options(options & ~Flag::Chatlist)
 , _selected(selected)
-, _limit(Limit(session)) {
+, _limit(Limit(session))
+, _chatlist(options & Flag::Chatlist) {
 }
 
 Main::Session &EditFilterChatsListController::session() const {
@@ -353,8 +354,11 @@ Main::Session &EditFilterChatsListController::session() const {
 }
 
 int EditFilterChatsListController::selectedTypesCount() const {
-	Expects(_typesDelegate != nullptr);
+	Expects(_chatlist || _typesDelegate != nullptr);
 
+	if (_chatlist) {
+		return 0;
+	}
 	auto result = 0;
 	for (auto i = 0; i != _typesDelegate->peerListFullRowsCount(); ++i) {
 		if (_typesDelegate->peerListRowAt(i)->checked()) {
@@ -396,7 +400,9 @@ bool EditFilterChatsListController::handleDeselectForeignRow(
 
 void EditFilterChatsListController::prepareViewHook() {
 	delegate()->peerListSetTitle(std::move(_title));
-	delegate()->peerListSetAboveWidget(prepareTypesList());
+	if (!_chatlist) {
+		delegate()->peerListSetAboveWidget(prepareTypesList());
+	}
 
 	const auto count = int(_peers.size());
 	const auto rows = std::make_unique<std::optional<ExceptionRow>[]>(count);
