@@ -7,8 +7,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
-#include <rpl/event_stream.h>
-#include "boxes/abstract_box.h"
+#include "ui/layers/box_content.h"
+
+namespace Settings {
+struct IconDescriptor;
+} // namespace Settings
 
 namespace style {
 struct SettingsCountButton;
@@ -42,7 +45,7 @@ public:
 		rpl::producer<QString> &&count,
 		Fn<void()> callback,
 		const style::SettingsCountButton &st,
-		const style::icon *icon = nullptr);
+		Settings::IconDescriptor &&descriptor);
 
 protected:
 	void prepare() override;
