@@ -138,7 +138,7 @@ void Row::elementAddRipple(
 	}
 	auto &ripple = *pointer;
 	if (!ripple) {
-		auto mask = Ui::RippleAnimation::roundRectMask(
+		auto mask = Ui::RippleAnimation::RoundRectMask(
 			(element == kAcceptButton
 				? _delegate->rowAcceptButtonSize()
 				: _delegate->rowRejectButtonSize()),
