@@ -46,6 +46,7 @@ struct InfographicDescriptor {
 	float64 premiumLimit = 0;
 	const style::icon *icon;
 	std::optional<tr::phrase<lngtag_count>> phrase;
+	bool complexRatio = false;
 };
 
 [[nodiscard]] rpl::producer<> BoxShowFinishes(not_null<Ui::GenericBox*> box) {
@@ -433,7 +434,11 @@ void SimpleLimitBox(
 		Ui::Premium::AddLimitRow(
 			top,
 			descriptor.premiumLimit,
-			descriptor.phrase);
+			descriptor.phrase,
+			0,
+			(descriptor.complexRatio
+				? (float64(descriptor.current) / descriptor.premiumLimit)
+				: Ui::Premium::kLimitRowRatio));
 		Settings::AddSkip(top, st::premiumInfographicPadding.bottom());
 	}
 
@@ -723,6 +728,50 @@ void FilterChatsLimitBox(
 		{ defaultLimit, current, premiumLimit, &st::premiumIconChats });
 }
 
+void FilterLinksLimitBox(
+		not_null<Ui::GenericBox*> box,
+		not_null<Main::Session*> session) {
+	const auto premium = session->premium();
+	const auto premiumPossible = session->premiumPossible();
+
+	const auto limits = Data::PremiumLimits(session);
+	const auto defaultLimit = float64(limits.dialogFiltersLinksDefault());
+	const auto premiumLimit = float64(limits.dialogFiltersLinksPremium());
+	const auto current = (premium ? premiumLimit : defaultLimit);
+
+	auto text = rpl::combine(
+		tr::lng_filter_links_limit1(
+			lt_count,
+			rpl::single(premium ? premiumLimit : defaultLimit),
+			Ui::Text::RichLangValue),
+		((premium || !premiumPossible)
+			? rpl::single(TextWithEntities())
+			: tr::lng_filter_links_limit2(
+				lt_count,
+				rpl::single(premiumLimit),
+				Ui::Text::RichLangValue))
+	) | rpl::map([](TextWithEntities &&a, TextWithEntities &&b) {
+		return b.text.isEmpty()
+			? a
+			: a.append(QChar(' ')).append(std::move(b));
+	});
+
+	SimpleLimitBox(
+		box,
+		session,
+		tr::lng_filter_links_limit_title(),
+		std::move(text),
+		"chatlist_invites",
+		{
+			defaultLimit,
+			current,
+			premiumLimit,
+			&st::premiumIconChats,
+			std::nullopt,
+			true });
+}
+
+
 void FiltersLimitBox(
 		not_null<Ui::GenericBox*> box,
 		not_null<Main::Session*> session) {
@@ -761,6 +810,50 @@ void FiltersLimitBox(
 		{ defaultLimit, current, premiumLimit, &st::premiumIconFolders });
 }
 
+void ShareableFiltersLimitBox(
+		not_null<Ui::GenericBox*> box,
+		not_null<Main::Session*> session) {
+	const auto premium = session->premium();
+	const auto premiumPossible = session->premiumPossible();
+
+	const auto limits = Data::PremiumLimits(session);
+	const auto defaultLimit = float64(limits.dialogShareableFiltersDefault());
+	const auto premiumLimit = float64(limits.dialogShareableFiltersPremium());
+	const auto current = float64(ranges::count_if(
+		session->data().chatsFilters().list(),
+		[](const Data::ChatFilter &f) { return f.chatlist(); }));
+
+	auto text = rpl::combine(
+		tr::lng_filter_shared_limit1(
+			lt_count,
+			rpl::single(premium ? premiumLimit : defaultLimit),
+			Ui::Text::RichLangValue),
+		((premium || !premiumPossible)
+			? rpl::single(TextWithEntities())
+			: tr::lng_filter_shared_limit2(
+				lt_count,
+				rpl::single(premiumLimit),
+				Ui::Text::RichLangValue))
+	) | rpl::map([](TextWithEntities &&a, TextWithEntities &&b) {
+		return b.text.isEmpty()
+			? a
+			: a.append(QChar(' ')).append(std::move(b));
+	});
+	SimpleLimitBox(
+		box,
+		session,
+		tr::lng_filter_shared_limit_title(),
+		std::move(text),
+		"chatlists_joined",
+		{
+			defaultLimit,
+			current,
+			premiumLimit,
+			&st::premiumIconFolders,
+			std::nullopt,
+			true });
+}
+
 void FilterPinsLimitBox(
 		not_null<Ui::GenericBox*> box,
 		not_null<Main::Session*> session,
