@@ -508,7 +508,8 @@ void SendFilesBox::refreshButtons() {
 			_send,
 			[=] { return _sendMenuType; },
 			[=] { sendSilent(); },
-			[=] { sendScheduled(); });
+			[=] { sendScheduled(); },
+			[=] { sendWhenOnline(); });
 	}
 	addButton(tr::lng_cancel(), [=] { closeBox(); });
 	_addFile = addLeftButton(
@@ -584,7 +585,8 @@ void SendFilesBox::addMenuButton() {
 				_menu.get(),
 				_sendMenuType,
 				[=] { sendSilent(); },
-				[=] { sendScheduled(); });
+				[=] { sendScheduled(); },
+				[=] { sendWhenOnline(); });
 		}
 		_menu->popup(QCursor::pos());
 		return true;
@@ -1091,13 +1093,13 @@ void SendFilesBox::captionResized() {
 }
 
 bool SendFilesBox::canAddFiles(not_null<const QMimeData*> data) const {
-	return CanAddUrls(base::GetMimeUrls(data)) || data->hasImage();
+	return data->hasImage() || CanAddUrls(Core::ReadMimeUrls(data));
 }
 
 bool SendFilesBox::addFiles(not_null<const QMimeData*> data) {
 	const auto premium = _controller->session().premium();
 	auto list = [&] {
-		const auto urls = base::GetMimeUrls(data);
+		const auto urls = Core::ReadMimeUrls(data);
 		auto result = CanAddUrls(urls)
 			? Storage::PrepareMediaList(
 				urls,
@@ -1390,4 +1392,8 @@ void SendFilesBox::sendScheduled() {
 		Ui::LayerOption::KeepOther);
 }
 
+void SendFilesBox::sendWhenOnline() {
+	send(Api::DefaultSendWhenOnlineOptions());
+}
+
 SendFilesBox::~SendFilesBox() = default;
