@@ -502,7 +502,8 @@ void ShareBox::showMenu(not_null<Ui::RpWidget*> parent) {
 		_menu.get(),
 		sendMenuType(),
 		[=] { submitSilent(); },
-		[=] { submitScheduled(); });
+		[=] { submitScheduled(); },
+		[=] { submitWhenOnline(); });
 	const auto success = (result == SendMenu::FillMenuResult::Success);
 	if (_descriptor.forwardOptions.show || success) {
 		_menu->setForcedVerticalOrigin(Ui::PopupMenu::VerticalOrigin::Bottom);
@@ -599,6 +600,10 @@ void ShareBox::submitScheduled() {
 		Ui::LayerOption::KeepOther);
 }
 
+void ShareBox::submitWhenOnline() {
+	submit(Api::DefaultSendWhenOnlineOptions());
+}
+
 void ShareBox::copyLink() {
 	if (const auto onstack = _descriptor.copyCallback) {
 		onstack();
@@ -1343,8 +1348,9 @@ ShareBox::SubmitCallback ShareBox::DefaultForwardCallback(
 		if (!state->requests.empty()) {
 			return; // Share clicked already.
 		}
-		auto items = history->owner().idsToItems(msgIds);
-		if (items.empty() || result.empty()) {
+		const auto items = history->owner().idsToItems(msgIds);
+		const auto existingIds = history->owner().itemsToIds(items);
+		if (existingIds.empty() || result.empty()) {
 			return;
 		}
 
@@ -1384,12 +1390,12 @@ ShareBox::SubmitCallback ShareBox::DefaultForwardCallback(
 				? Flag::f_drop_media_captions
 				: Flag(0));
 		auto mtpMsgIds = QVector<MTPint>();
-		mtpMsgIds.reserve(msgIds.size());
-		for (const auto &fullId : msgIds) {
+		mtpMsgIds.reserve(existingIds.size());
+		for (const auto &fullId : existingIds) {
 			mtpMsgIds.push_back(MTP_int(fullId.msg));
 		}
 		const auto generateRandom = [&] {
-			auto result = QVector<MTPlong>(msgIds.size());
+			auto result = QVector<MTPlong>(existingIds.size());
 			for (auto &value : result) {
 				value = base::RandomValue<MTPlong>();
 			}
