@@ -64,6 +64,7 @@ public:
 	[[nodiscard]] bool isActive() const;
 	void showAndActivate();
 	void minimize();
+	void toggleFullScreen();
 	void replaceCall(not_null<Call*> call);
 	void closeBeforeDestroy();
 
@@ -77,6 +78,8 @@ public:
 		bool withAudio) override;
 	void chooseSourceStop() override;
 
+	[[nodiscard]] rpl::producer<bool> startOutgoingRequests() const;
+
 	[[nodiscard]] rpl::lifetime &lifetime();
 
 private:
@@ -87,6 +90,7 @@ private:
 		Answer,
 		Hangup,
 		Redial,
+		StartCall,
 	};
 
 	[[nodiscard]] not_null<Ui::RpWindow*> window() const;
@@ -147,9 +151,10 @@ private:
 	bool _outgoingPreviewInBody = false;
 	std::optional<AnswerHangupRedialState> _answerHangupRedialState;
 	Ui::Animations::Simple _hangupShownProgress;
-	object_ptr<Ui::CallButton> _screencast;
+	object_ptr<Ui::FadeWrap<Ui::CallButton>> _screencast;
 	object_ptr<Ui::CallButton> _camera;
-	object_ptr<Ui::CallButton> _mute;
+	base::unique_qptr<Ui::CallButton> _startVideo;
+	object_ptr<Ui::FadeWrap<Ui::CallButton>> _mute;
 	object_ptr<Ui::FlatLabel> _name;
 	object_ptr<Ui::FlatLabel> _status;
 	object_ptr<Ui::RpWidget> _fingerprint = { nullptr };
@@ -163,6 +168,8 @@ private:
 	base::Timer _updateDurationTimer;
 	base::Timer _updateOuterRippleTimer;
 
+	rpl::event_stream<bool> _startOutgoingRequests;
+
 };
 
 } // namespace Calls
