@@ -365,10 +365,8 @@ inline int indexOfInFirstN(const T &v, const U &elem, int last) {
 }
 
 FieldAutocomplete::StickerRows FieldAutocomplete::getStickerSuggestions() {
-	const auto list = _controller->session().data().stickers().getListByEmoji(
-		_emoji,
-		_stickersSeed
-	);
+	const auto data = &_controller->session().data().stickers();
+	const auto list = data->getListByEmoji({ _emoji }, _stickersSeed);
 	auto result = ranges::views::all(
 		list
 	) | ranges::views::transform([](not_null<DocumentData*> sticker) {
@@ -1285,7 +1283,8 @@ void FieldAutocomplete::Inner::contextMenuEvent(QContextMenuEvent *e) {
 		_menu,
 		type,
 		SendMenu::DefaultSilentCallback(send),
-		SendMenu::DefaultScheduleCallback(this, type, send));
+		SendMenu::DefaultScheduleCallback(this, type, send),
+		SendMenu::DefaultWhenOnlineCallback(send));
 
 	if (!_menu->empty()) {
 		_menu->popup(QCursor::pos());
