@@ -15,6 +15,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/platform/base_platform_info.h"
 #include "webrtc/webrtc_create_adm.h"
 #include "media/player/media_player_instance.h"
+#include "media/media_common.h"
 #include "ui/gl/gl_detection.h"
 #include "calls/group/calls_group_common.h"
 #include "spellcheck/spellcheck_types.h"
@@ -197,7 +198,9 @@ QByteArray Settings::serialize() const {
 		+ sizeof(qint32)
 		+ sizeof(quint64)
 		+ sizeof(qint32) * 3
-		+ Serialize::bytearraySize(mediaViewPosition);
+		+ Serialize::bytearraySize(mediaViewPosition)
+		+ sizeof(qint32)
+		+ sizeof(quint64);
 
 	auto result = QByteArray();
 	result.reserve(size);
@@ -213,7 +216,7 @@ QByteArray Settings::serialize() const {
 			<< qint32(_askDownloadPath ? 1 : 0)
 			<< _downloadPath.current()
 			<< _downloadPathBookmark
-			<< qint32(_nonDefaultVoicePlaybackSpeed ? 1 : 0)
+			<< qint32(1)
 			<< qint32(_soundNotify ? 1 : 0)
 			<< qint32(_desktopNotify ? 1 : 0)
 			<< qint32(_flashBounceNotify ? 1 : 0)
@@ -245,7 +248,7 @@ QByteArray Settings::serialize() const {
 			<< qint32(_suggestEmoji ? 1 : 0)
 			<< qint32(_suggestStickersByEmoji ? 1 : 0)
 			<< qint32(_spellcheckerEnabled.current() ? 1 : 0)
-			<< qint32(SerializePlaybackSpeed(_videoPlaybackSpeed.current()))
+			<< qint32(SerializePlaybackSpeed(_videoPlaybackSpeed))
 			<< _videoPipGeometry
 			<< qint32(_dictionariesEnabled.current().size());
 		for (const auto i : _dictionariesEnabled.current()) {
@@ -293,7 +296,7 @@ QByteArray Settings::serialize() const {
 			<< qint32(_disableOpenGL ? 1 : 0)
 			<< _photoEditorBrush
 			<< qint32(_groupCallNoiseSuppression ? 1 : 0)
-			<< qint32(_voicePlaybackSpeed * 100)
+			<< qint32(SerializePlaybackSpeed(_voicePlaybackSpeed))
 			<< qint32(_closeToTaskbar.current() ? 1 : 0)
 			<< _customDeviceModel.current()
 			<< qint32(_playerRepeatMode.current())
@@ -329,7 +332,9 @@ QByteArray Settings::serialize() const {
 			<< qint32(_windowTitleContent.current().hideChatName ? 1 : 0)
 			<< qint32(_windowTitleContent.current().hideAccountName ? 1 : 0)
 			<< qint32(_windowTitleContent.current().hideTotalUnread ? 1 : 0)
-			<< mediaViewPosition;
+			<< mediaViewPosition
+			<< qint32(_ignoreBatterySaving.current() ? 1 : 0)
+			<< quint64(_macRoundIconDigest.value_or(0));
 	}
 	return result;
 }
@@ -350,7 +355,7 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	qint32 askDownloadPath = _askDownloadPath ? 1 : 0;
 	QString downloadPath = _downloadPath.current();
 	QByteArray downloadPathBookmark = _downloadPathBookmark;
-	qint32 nonDefaultVoicePlaybackSpeed = _nonDefaultVoicePlaybackSpeed ? 1 : 0;
+	qint32 nonDefaultVoicePlaybackSpeed = 1;
 	qint32 soundNotify = _soundNotify ? 1 : 0;
 	qint32 desktopNotify = _desktopNotify ? 1 : 0;
 	qint32 flashBounceNotify = _flashBounceNotify ? 1 : 0;
@@ -380,8 +385,8 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	qint32 suggestEmoji = _suggestEmoji ? 1 : 0;
 	qint32 suggestStickersByEmoji = _suggestStickersByEmoji ? 1 : 0;
 	qint32 spellcheckerEnabled = _spellcheckerEnabled.current() ? 1 : 0;
-	qint32 videoPlaybackSpeed = Core::Settings::SerializePlaybackSpeed(_videoPlaybackSpeed.current());
-	qint32 voicePlaybackSpeed = _voicePlaybackSpeed * 100;
+	qint32 videoPlaybackSpeed = SerializePlaybackSpeed(_videoPlaybackSpeed);
+	qint32 voicePlaybackSpeed = SerializePlaybackSpeed(_voicePlaybackSpeed);
 	QByteArray videoPipGeometry = _videoPipGeometry;
 	qint32 dictionariesEnabledCount = 0;
 	std::vector<int> dictionariesEnabled;
@@ -433,6 +438,8 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	qint32 hideAccountName = _windowTitleContent.current().hideAccountName ? 1 : 0;
 	qint32 hideTotalUnread = _windowTitleContent.current().hideTotalUnread ? 1 : 0;
 	QByteArray mediaViewPosition;
+	qint32 ignoreBatterySaving = _ignoreBatterySaving.current() ? 1 : 0;
+	quint64 macRoundIconDigest = _macRoundIconDigest.value_or(0);
 
 	stream >> themesAccentColors;
 	if (!stream.atEnd()) {
@@ -661,6 +668,12 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	if (!stream.atEnd()) {
 		stream >> mediaViewPosition;
 	}
+	if (!stream.atEnd()) {
+		stream >> ignoreBatterySaving;
+	}
+	if (!stream.atEnd()) {
+		stream >> macRoundIconDigest;
+	}
 	if (stream.status() != QDataStream::Ok) {
 		LOG(("App Error: "
 			"Bad data for Core::Settings::constructFromSerialized()"));
@@ -730,16 +743,9 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	_suggestStickersByEmoji = (suggestStickersByEmoji == 1);
 	_spellcheckerEnabled = (spellcheckerEnabled == 1);
 	_videoPlaybackSpeed = DeserializePlaybackSpeed(videoPlaybackSpeed);
-	{
-		// Restore settings from 3.0.1 version.
-		if (voicePlaybackSpeed == 100) {
-			_nonDefaultVoicePlaybackSpeed = false;
-			_voicePlaybackSpeed = 2.0;
-		} else {
-			_nonDefaultVoicePlaybackSpeed =
-				(nonDefaultVoicePlaybackSpeed == 1);
-			_voicePlaybackSpeed = voicePlaybackSpeed / 100.;
-		}
+	_voicePlaybackSpeed = DeserializePlaybackSpeed(voicePlaybackSpeed);
+	if (nonDefaultVoicePlaybackSpeed != 1) {
+		_voicePlaybackSpeed.enabled = false;
 	}
 	_videoPipGeometry = (videoPipGeometry);
 	_dictionariesEnabled = std::move(dictionariesEnabled);
@@ -804,17 +810,17 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 	_closeToTaskbar = (closeToTaskbar == 1);
 	_customDeviceModel = customDeviceModel;
 	_accountsOrder = accountsOrder;
-	const auto uncheckedPlayerRepeatMode = static_cast<Media::Player::RepeatMode>(playerRepeatMode);
+	const auto uncheckedPlayerRepeatMode = static_cast<Media::RepeatMode>(playerRepeatMode);
 	switch (uncheckedPlayerRepeatMode) {
-	case Media::Player::RepeatMode::None:
-	case Media::Player::RepeatMode::One:
-	case Media::Player::RepeatMode::All: _playerRepeatMode = uncheckedPlayerRepeatMode; break;
+	case Media::RepeatMode::None:
+	case Media::RepeatMode::One:
+	case Media::RepeatMode::All: _playerRepeatMode = uncheckedPlayerRepeatMode; break;
 	}
-	const auto uncheckedPlayerOrderMode = static_cast<Media::Player::OrderMode>(playerOrderMode);
+	const auto uncheckedPlayerOrderMode = static_cast<Media::OrderMode>(playerOrderMode);
 	switch (uncheckedPlayerOrderMode) {
-	case Media::Player::OrderMode::Default:
-	case Media::Player::OrderMode::Reverse:
-	case Media::Player::OrderMode::Shuffle: _playerOrderMode = uncheckedPlayerOrderMode; break;
+	case Media::OrderMode::Default:
+	case Media::OrderMode::Reverse:
+	case Media::OrderMode::Shuffle: _playerOrderMode = uncheckedPlayerOrderMode; break;
 	}
 	_macWarnBeforeQuit = (macWarnBeforeQuit == 1);
 	_hardwareAcceleratedVideo = (hardwareAcceleratedVideo == 1);
@@ -857,6 +863,8 @@ void Settings::addFromSerialized(const QByteArray &serialized) {
 			_mediaViewPosition = { .maximized = 2 };
 		}
 	}
+	_ignoreBatterySaving = (ignoreBatterySaving == 1);
+	_macRoundIconDigest = macRoundIconDigest ? macRoundIconDigest : std::optional<uint64>();
 }
 
 QString Settings::getSoundPath(const QString &key) const {
@@ -1087,7 +1095,6 @@ void Settings::resetOnLastLogout() {
 	_downloadPath = QString();
 	_downloadPathBookmark = QByteArray();
 
-	_nonDefaultVoicePlaybackSpeed = false;
 	_soundNotify = true;
 	_desktopNotify = true;
 	_flashBounceNotify = true;
@@ -1131,8 +1138,8 @@ void Settings::resetOnLastLogout() {
 	_suggestStickersByEmoji = true;
 	_suggestAnimatedEmoji = true;
 	_spellcheckerEnabled = true;
-	_videoPlaybackSpeed = 1.;
-	_voicePlaybackSpeed = 1.;
+	_videoPlaybackSpeed = PlaybackSpeed();
+	_voicePlaybackSpeed = PlaybackSpeed();
 	//_videoPipGeometry = QByteArray();
 	_dictionariesEnabled = std::vector<int>();
 	_autoDownloadDictionaries = true;
@@ -1166,6 +1173,34 @@ float64 Settings::DefaultDialogsWidthRatio() {
 		: kDefaultDialogsWidthRatio;
 }
 
+qint32 Settings::SerializePlaybackSpeed(PlaybackSpeed speed) {
+	using namespace Media;
+
+	const auto value = int(base::SafeRound(
+		std::clamp(speed.value, kSpeedMin, kSpeedMax) * 100));
+	return speed.enabled ? value : -value;
+}
+
+auto Settings::DeserializePlaybackSpeed(qint32 speed) -> PlaybackSpeed {
+	using namespace Media;
+
+	auto enabled = true;
+	const auto validate = [&](float64 result) {
+		return PlaybackSpeed{
+			.value = (result == 1.) ? kSpedUpDefault : result,
+			.enabled = enabled && (result != 1.),
+		};
+	};
+	if (speed >= 0 && speed < 10) {
+		// The old values in settings.
+		return validate((std::clamp(speed, 0, 6) + 2) / 4.);
+	} else if (speed < 0) {
+		speed = -speed;
+		enabled = false;
+	}
+	return validate(std::clamp(speed / 100., kSpeedMin, kSpeedMax));
+}
+
 void Settings::setTranslateButtonEnabled(bool value) {
 	_translateButtonEnabled = value;
 }
