@@ -10,6 +10,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_authorizations.h"
 #include "api/api_confirm_phone.h"
 #include "api/api_text_entities.h"
+#include "api/api_chat_filters.h"
 #include "api/api_chat_invite.h"
 #include "base/qthelp_regex.h"
 #include "base/qthelp_url.h"
@@ -78,6 +79,18 @@ bool JoinGroupByHash(
 	return true;
 }
 
+bool JoinFilterBySlug(
+		Window::SessionController *controller,
+		const Match &match,
+		const QVariant &context) {
+	if (!controller) {
+		return false;
+	}
+	Api::CheckFilterInvite(controller, match->captured(1));
+	controller->window().activate();
+	return true;
+}
+
 bool ShowStickerSet(
 		Window::SessionController *controller,
 		const Match &match,
@@ -243,14 +256,12 @@ bool ShowPassportForm(
 	const auto nonce = params.value(
 		Passport::NonceNameByScope(scope),
 		QString());
-	const auto errors = params.value("errors", QString());
 	controller->showPassportForm(Passport::FormRequest(
 		botId,
 		scope,
 		callback,
 		publicKey,
-		nonce,
-		errors));
+		nonce));
 	return true;
 }
 
@@ -373,6 +384,8 @@ bool ResolveUsernameOrPhone(
 	if (const auto postId = postParam.toInt()) {
 		post = postId;
 	}
+	const auto appname = params.value(u"appname"_q);
+	const auto appstart = params.value(u"startapp"_q);
 	const auto commentParam = params.value(u"comment"_q);
 	const auto commentId = commentParam.toInt();
 	const auto topicParam = params.value(u"topic"_q);
@@ -384,6 +397,12 @@ bool ResolveUsernameOrPhone(
 		startToken = gameParam;
 		resolveType = ResolveType::ShareGame;
 	}
+	if (startToken.isEmpty() && params.contains(u"startapp"_q)) {
+		startToken = params.value(u"startapp"_q);
+	}
+	if (!appname.isEmpty()) {
+		resolveType = ResolveType::BotApp;
+	}
 	const auto myContext = context.value<ClickHandlerContext>();
 	using Navigation = Window::SessionNavigation;
 	controller->showPeerByLink(Navigation::PeerByLinkInfo{
@@ -403,6 +422,8 @@ bool ResolveUsernameOrPhone(
 		.startToken = startToken,
 		.startAdminRights = adminRights,
 		.startAutoSubmit = myContext.botStartAutoSubmit,
+		.botAppName = appname.isEmpty() ? postParam : appname,
+		.botAppForceConfirmation = myContext.mayShowConfirmation,
 		.attachBotUsername = params.value(u"attach"_q),
 		.attachBotToggleCommand = (params.contains(u"startattach"_q)
 			? params.value(u"startattach"_q)
@@ -743,7 +764,10 @@ bool ResolveTestChatTheme(
 			}
 			const auto recache = [&](Data::CloudThemeType type) {
 				[[maybe_unused]] auto value = theme->settings.contains(type)
-					? controller->cachedChatThemeValue(*theme, type)
+					? controller->cachedChatThemeValue(
+						*theme,
+						Data::WallPaper(0),
+						type)
 					: nullptr;
 			};
 			recache(Data::CloudThemeType::Dark);
@@ -821,6 +845,10 @@ const std::vector<LocalUrlHandler> &LocalUrlHandlers() {
 			u"^join/?\\?invite=([a-zA-Z0-9\\.\\_\\-]+)(&|$)"_q,
 			JoinGroupByHash
 		},
+		{
+			u"^addlist/?\\?slug=([a-zA-Z0-9\\.\\_\\-]+)(&|$)"_q,
+			JoinFilterBySlug
+		},
 		{
 			u"^(addstickers|addemoji)/?\\?set=([a-zA-Z0-9\\.\\_]+)(&|$)"_q,
 			ShowStickerSet
@@ -945,6 +973,8 @@ QString TryConvertUrlToLocal(QString url) {
 			return u"tg://resolve?phone="_q + phoneMatch->captured(1) + (params.isEmpty() ? QString() : '&' + params);
 		} else if (const auto joinChatMatch = regex_match(u"^(joinchat/|\\+|\\%20)([a-zA-Z0-9\\.\\_\\-]+)(\\?|$)"_q, query, matchOptions)) {
 			return u"tg://join?invite="_q + url_encode(joinChatMatch->captured(2));
+		} else if (const auto joinFilterMatch = regex_match(u"^(addlist/)([a-zA-Z0-9\\.\\_\\-]+)(\\?|$)"_q, query, matchOptions)) {
+			return u"tg://addlist?slug="_q + url_encode(joinFilterMatch->captured(2));
 		} else if (const auto stickerSetMatch = regex_match(u"^(addstickers|addemoji)/([a-zA-Z0-9\\.\\_]+)(\\?|$)"_q, query, matchOptions)) {
 			return u"tg://"_q + stickerSetMatch->captured(1) + "?set=" + url_encode(stickerSetMatch->captured(2));
 		} else if (const auto themeMatch = regex_match(u"^addtheme/([a-zA-Z0-9\\.\\_]+)(\\?|$)"_q, query, matchOptions)) {
@@ -1004,6 +1034,7 @@ QString TryConvertUrlToLocal(QString url) {
 			"("
 				"/?\\?|"
 				"/?$|"
+				"/[a-zA-Z0-9\\.\\_]+/?(\\?|$)|"
 				"/\\d+/?(\\?|$)|"
 				"/\\d+/\\d+/?(\\?|$)"
 			")"_q, query, matchOptions)) {
@@ -1014,6 +1045,8 @@ QString TryConvertUrlToLocal(QString url) {
 				added = u"&topic=%1&post=%2"_q.arg(threadPostMatch->captured(1)).arg(threadPostMatch->captured(2));
 			} else if (const auto postMatch = regex_match(u"^/(\\d+)(/?\\?|/?$)"_q, usernameMatch->captured(2))) {
 				added = u"&post="_q + postMatch->captured(1);
+			} else if (const auto appNameMatch = regex_match(u"^/([a-zA-Z0-9\\.\\_]+)(/?\\?|/?$)"_q, usernameMatch->captured(2))) {
+				added = u"&appname="_q + appNameMatch->captured(1);
 			}
 			return base + added + (params.isEmpty() ? QString() : '&' + params);
 		}
