@@ -56,52 +56,54 @@ struct PeerUpdate {
 		None = 0,
 
 		// Common flags
-		Name              = (1ULL << 0),
-		Username          = (1ULL << 1),
-		Photo             = (1ULL << 2),
-		About             = (1ULL << 3),
-		Notifications     = (1ULL << 4),
-		Migration         = (1ULL << 5),
-		UnavailableReason = (1ULL << 6),
-		ChatThemeEmoji    = (1ULL << 7),
-		IsBlocked         = (1ULL << 8),
-		MessagesTTL       = (1ULL << 9),
-		FullInfo          = (1ULL << 10),
-		Usernames         = (1ULL << 11),
+		Name                = (1ULL << 0),
+		Username            = (1ULL << 1),
+		Photo               = (1ULL << 2),
+		About               = (1ULL << 3),
+		Notifications       = (1ULL << 4),
+		Migration           = (1ULL << 5),
+		UnavailableReason   = (1ULL << 6),
+		ChatThemeEmoji      = (1ULL << 7),
+		ChatWallPaper       = (1ULL << 8),
+		IsBlocked           = (1ULL << 9),
+		MessagesTTL         = (1ULL << 10),
+		FullInfo            = (1ULL << 11),
+		Usernames           = (1ULL << 12),
+		TranslationDisabled = (1ULL << 13),
 
 		// For users
-		CanShareContact   = (1ULL << 12),
-		IsContact         = (1ULL << 13),
-		PhoneNumber       = (1ULL << 14),
-		OnlineStatus      = (1ULL << 15),
-		BotCommands       = (1ULL << 16),
-		BotCanBeInvited   = (1ULL << 17),
-		BotStartToken     = (1ULL << 18),
-		CommonChats       = (1ULL << 19),
-		HasCalls          = (1ULL << 20),
-		SupportInfo       = (1ULL << 21),
-		IsBot             = (1ULL << 22),
-		EmojiStatus       = (1ULL << 23),
+		CanShareContact     = (1ULL << 14),
+		IsContact           = (1ULL << 15),
+		PhoneNumber         = (1ULL << 16),
+		OnlineStatus        = (1ULL << 17),
+		BotCommands         = (1ULL << 18),
+		BotCanBeInvited     = (1ULL << 19),
+		BotStartToken       = (1ULL << 20),
+		CommonChats         = (1ULL << 21),
+		HasCalls            = (1ULL << 22),
+		SupportInfo         = (1ULL << 23),
+		IsBot               = (1ULL << 24),
+		EmojiStatus         = (1ULL << 25),
 
 		// For chats and channels
-		InviteLinks       = (1ULL << 24),
-		Members           = (1ULL << 25),
-		Admins            = (1ULL << 26),
-		BannedUsers       = (1ULL << 27),
-		Rights            = (1ULL << 28),
-		PendingRequests   = (1ULL << 29),
-		Reactions         = (1ULL << 30),
+		InviteLinks         = (1ULL << 26),
+		Members             = (1ULL << 27),
+		Admins              = (1ULL << 28),
+		BannedUsers         = (1ULL << 29),
+		Rights              = (1ULL << 30),
+		PendingRequests     = (1ULL << 31),
+		Reactions           = (1ULL << 32),
 
 		// For channels
-		ChannelAmIn       = (1ULL << 31),
-		StickersSet       = (1ULL << 32),
-		ChannelLinkedChat = (1ULL << 33),
-		ChannelLocation   = (1ULL << 34),
-		Slowmode          = (1ULL << 35),
-		GroupCall         = (1ULL << 36),
+		ChannelAmIn         = (1ULL << 33),
+		StickersSet         = (1ULL << 34),
+		ChannelLinkedChat   = (1ULL << 35),
+		ChannelLocation     = (1ULL << 36),
+		Slowmode            = (1ULL << 37),
+		GroupCall           = (1ULL << 38),
 
 		// For iteration
-		LastUsedBit       = (1ULL << 36),
+		LastUsedBit         = (1ULL << 38),
 	};
 	using Flags = base::flags<Flag>;
 	friend inline constexpr auto is_flag_type(Flag) { return true; }
@@ -128,8 +130,10 @@ struct HistoryUpdate {
 		OutboxRead         = (1U << 10),
 		BotKeyboard        = (1U << 11),
 		CloudDraft         = (1U << 12),
+		TranslateFrom      = (1U << 13),
+		TranslatedTo       = (1U << 14),
 
-		LastUsedBit        = (1U << 12),
+		LastUsedBit        = (1U << 14),
 	};
 	using Flags = base::flags<Flag>;
 	friend inline constexpr auto is_flag_type(Flag) { return true; }
