@@ -392,7 +392,7 @@ const Invoice *Media::invoice() const {
 	return nullptr;
 }
 
-Data::CloudImage *Media::location() const {
+CloudImage *Media::location() const {
 	return nullptr;
 }
 
@@ -400,6 +400,10 @@ PollData *Media::poll() const {
 	return nullptr;
 }
 
+const WallPaper *Media::paper() const {
+	return nullptr;
+}
+
 bool Media::uploading() const {
 	return false;
 }
@@ -1238,7 +1242,7 @@ std::unique_ptr<Media> MediaLocation::clone(not_null<HistoryItem*> parent) {
 		_description);
 }
 
-Data::CloudImage *MediaLocation::location() const {
+CloudImage *MediaLocation::location() const {
 	return _location;
 }
 
@@ -1827,7 +1831,7 @@ ClickHandlerPtr MediaDice::MakeHandler(
 			.durationMs = Ui::Toast::kDefaultDuration * 2,
 			.multiline = true,
 		};
-		if (Data::CanSend(history->peer, ChatRestriction::SendOther)) {
+		if (CanSend(history->peer, ChatRestriction::SendOther)) {
 			auto link = Ui::Text::Link(
 				tr::lng_about_random_send(tr::now).toUpper());
 			link.entities.push_back(
@@ -1883,10 +1887,6 @@ int MediaGiftBox::months() const {
 	return _months;
 }
 
-bool MediaGiftBox::allowsRevoke(TimeId now) const {
-	return false;
-}
-
 TextWithEntities MediaGiftBox::notificationText() const {
 	return {};
 }
@@ -1899,10 +1899,6 @@ TextForMimeData MediaGiftBox::clipboardText() const {
 	return {};
 }
 
-bool MediaGiftBox::forceForwardedInfo() const {
-	return false;
-}
-
 bool MediaGiftBox::updateInlineResultMedia(const MTPMessageMedia &media) {
 	return false;
 }
@@ -1928,4 +1924,50 @@ void MediaGiftBox::setActivated(bool activated) {
 	_activated = activated;
 }
 
+MediaWallPaper::MediaWallPaper(
+	not_null<HistoryItem*> parent,
+	const WallPaper &paper)
+: Media(parent)
+, _paper(paper) {
+}
+
+MediaWallPaper::~MediaWallPaper() = default;
+
+std::unique_ptr<Media> MediaWallPaper::clone(not_null<HistoryItem*> parent) {
+	return std::make_unique<MediaWallPaper>(parent, _paper);
+}
+
+const WallPaper *MediaWallPaper::paper() const {
+	return &_paper;
+}
+
+TextWithEntities MediaWallPaper::notificationText() const {
+	return {};
+}
+
+QString MediaWallPaper::pinnedTextSubstring() const {
+	return {};
+}
+
+TextForMimeData MediaWallPaper::clipboardText() const {
+	return {};
+}
+
+bool MediaWallPaper::updateInlineResultMedia(const MTPMessageMedia &media) {
+	return false;
+}
+
+bool MediaWallPaper::updateSentMedia(const MTPMessageMedia &media) {
+	return false;
+}
+
+std::unique_ptr<HistoryView::Media> MediaWallPaper::createView(
+		not_null<HistoryView::Element*> message,
+		not_null<HistoryItem*> realParent,
+		HistoryView::Element *replacing) {
+	return std::make_unique<HistoryView::ServiceBox>(
+		message,
+		std::make_unique<HistoryView::ThemeDocumentBox>(message, _paper));
+}
+
 } // namespace Data
