@@ -8,6 +8,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #pragma once
 
 #include "data/data_location.h"
+#include "data/data_wall_paper.h"
 
 class Image;
 class History;
@@ -35,6 +36,7 @@ struct ToPreviewOptions;
 namespace Data {
 
 class CloudImage;
+class WallPaper;
 
 enum class CallFinishReason : char {
 	Missed,
@@ -105,8 +107,9 @@ public:
 	virtual const Call *call() const;
 	virtual GameData *game() const;
 	virtual const Invoice *invoice() const;
-	virtual Data::CloudImage *location() const;
+	virtual CloudImage *location() const;
 	virtual PollData *poll() const;
+	virtual const WallPaper *paper() const;
 
 	virtual bool uploading() const;
 	virtual Storage::SharedMediaTypesMask sharedMediaTypes() const;
@@ -128,7 +131,6 @@ public:
 	virtual bool forwardedBecomesUnread() const;
 	virtual bool dropForwardedInfo() const;
 	virtual bool forceForwardedInfo() const;
-	virtual QString errorTextForForward(not_null<PeerData*> peer) const;
 	[[nodiscard]] virtual bool hasSpoiler() const;
 
 	[[nodiscard]] virtual bool consumeMessageText(
@@ -190,7 +192,6 @@ public:
 	TextForMimeData clipboardText() const override;
 	bool allowsEditCaption() const override;
 	bool allowsEditMedia() const override;
-	QString errorTextForForward(not_null<PeerData*> peer) const override;
 	bool hasSpoiler() const override;
 
 	bool updateInlineResultMedia(const MTPMessageMedia &media) override;
@@ -234,7 +235,6 @@ public:
 	bool allowsEditMedia() const override;
 	bool forwardedBecomesUnread() const override;
 	bool dropForwardedInfo() const override;
-	QString errorTextForForward(not_null<PeerData*> peer) const override;
 	bool hasSpoiler() const override;
 
 	bool updateInlineResultMedia(const MTPMessageMedia &media) override;
@@ -294,7 +294,7 @@ public:
 
 	std::unique_ptr<Media> clone(not_null<HistoryItem*> parent) override;
 
-	Data::CloudImage *location() const override;
+	CloudImage *location() const override;
 	ItemPreview toPreview(ToPreviewOptions options) const override;
 	TextWithEntities notificationText() const override;
 	QString pinnedTextSubstring() const override;
@@ -309,7 +309,7 @@ public:
 
 private:
 	LocationPoint _point;
-	not_null<Data::CloudImage*> _location;
+	not_null<CloudImage*> _location;
 	QString _title;
 	QString _description;
 
@@ -395,7 +395,6 @@ public:
 	TextWithEntities notificationText() const override;
 	QString pinnedTextSubstring() const override;
 	TextForMimeData clipboardText() const override;
-	QString errorTextForForward(not_null<PeerData*> peer) const override;
 	bool dropForwardedInfo() const override;
 
 	bool consumeMessageText(const TextWithEntities &text) override;
@@ -460,7 +459,6 @@ public:
 	TextWithEntities notificationText() const override;
 	QString pinnedTextSubstring() const override;
 	TextForMimeData clipboardText() const override;
-	QString errorTextForForward(not_null<PeerData*> peer) const override;
 
 	bool updateInlineResultMedia(const MTPMessageMedia &media) override;
 	bool updateSentMedia(const MTPMessageMedia &media) override;
@@ -522,11 +520,9 @@ public:
 	[[nodiscard]] bool activated() const;
 	void setActivated(bool activated);
 
-	bool allowsRevoke(TimeId now) const override;
 	TextWithEntities notificationText() const override;
 	QString pinnedTextSubstring() const override;
 	TextForMimeData clipboardText() const override;
-	bool forceForwardedInfo() const override;
 
 	bool updateInlineResultMedia(const MTPMessageMedia &media) override;
 	bool updateSentMedia(const MTPMessageMedia &media) override;
@@ -542,6 +538,31 @@ private:
 
 };
 
+class MediaWallPaper final : public Media {
+public:
+	MediaWallPaper(not_null<HistoryItem*> parent, const WallPaper &paper);
+	~MediaWallPaper();
+
+	std::unique_ptr<Media> clone(not_null<HistoryItem*> parent) override;
+
+	const WallPaper *paper() const override;
+
+	TextWithEntities notificationText() const override;
+	QString pinnedTextSubstring() const override;
+	TextForMimeData clipboardText() const override;
+
+	bool updateInlineResultMedia(const MTPMessageMedia &media) override;
+	bool updateSentMedia(const MTPMessageMedia &media) override;
+	std::unique_ptr<HistoryView::Media> createView(
+		not_null<HistoryView::Element*> message,
+		not_null<HistoryItem*> realParent,
+		HistoryView::Element *replacing = nullptr) override;
+
+private:
+	const WallPaper _paper;
+
+};
+
 [[nodiscard]] TextForMimeData WithCaptionClipboardText(
 	const QString &attachType,
 	TextForMimeData &&caption);
