@@ -37,6 +37,7 @@ class ForumTopic;
 class Session;
 class GroupCall;
 struct ReactionId;
+class WallPaper;
 
 [[nodiscard]] int PeerColorIndex(PeerId peerId);
 
@@ -403,6 +404,9 @@ public:
 	void setThemeEmoji(const QString &emoticon);
 	[[nodiscard]] const QString &themeEmoji() const;
 
+	void setWallPaper(std::optional<Data::WallPaper> paper);
+	[[nodiscard]] const Data::WallPaper *wallPaper() const;
+
 	const PeerId id;
 	MTPinputPeer input = MTP_inputPeerEmpty();
 
@@ -457,6 +461,7 @@ private:
 
 	QString _about;
 	QString _themeEmoticon;
+	std::unique_ptr<Data::WallPaper> _wallPaper;
 
 };
 
