@@ -1030,8 +1030,14 @@ auto HtmlWriter::Wrap::pushMessage(
 	}, [&](const ActionCustomAction &data) {
 		return data.message;
 	}, [&](const ActionBotAllowed &data) {
-		return "You allowed this bot to message you when you logged in on "
-			+ SerializeString(data.domain);
+		return data.attachMenu
+			? "You allowed this bot to message you "
+			"when you added it in the attachment menu."_q
+			: data.app.isEmpty()
+			? ("You allowed this bot to message you when you opened "
+				+ SerializeString(data.app))
+			: ("You allowed this bot to message you when you logged in on "
+				+ SerializeString(data.domain));
 	}, [&](const ActionSecureValuesSent &data) {
 		auto list = std::vector<QByteArray>();
 		for (const auto type : data.types) {
@@ -1165,11 +1171,15 @@ auto HtmlWriter::Wrap::pushMessage(
 		return serviceFrom + " changed topic " + parts.join(',');
 	}, [&](const ActionSuggestProfilePhoto &data) {
 		return serviceFrom + " suggests to use this photo";
-	}, [&](const ActionAttachMenuBotAllowed &data) {
-		return "You allowed this bot to message you "
-			"when you added it in the attachment menu."_q;
 	}, [&](const ActionRequestedPeer &data) {
 		return "requested: "_q/* + data.peerId*/;
+	}, [&](const ActionSetChatWallPaper &data) {
+		return serviceFrom + " set a new background for this chat";
+	}, [&](const ActionSetSameChatWallPaper &data) {
+		return serviceFrom
+			+ " set "
+			+ wrapReplyToLink("the same background")
+			+ " for this chat";
 	}, [](v::null_t) { return QByteArray(); });
 
 	if (!serviceText.isEmpty()) {
