@@ -475,8 +475,16 @@ QByteArray SerializeMessage(
 		pushActor();
 		push("information_text", data.message);
 	}, [&](const ActionBotAllowed &data) {
-		pushAction("allow_sending_messages");
-		push("reason_domain", data.domain);
+		if (data.attachMenu) {
+			pushAction("attach_menu_bot_allowed");
+		} else if (data.appId) {
+			pushAction("allow_sending_messages");
+			push("reason_app_id", data.appId);
+			push("reason_app_name", data.app);
+		} else {
+			pushAction("allow_sending_messages");
+			push("reason_domain", data.domain);
+		}
 	}, [&](const ActionSecureValuesSent &data) {
 		pushAction("send_passport_values");
 		auto list = std::vector<QByteArray>();
@@ -577,14 +585,18 @@ QByteArray SerializeMessage(
 		pushActor();
 		pushAction("suggest_profile_photo");
 		pushPhoto(data.photo.image);
-	}, [&](const ActionAttachMenuBotAllowed &data) {
-		pushActor();
-		pushAction("attach_menu_bot_allowed");
 	}, [&](const ActionRequestedPeer &data) {
 		pushActor();
 		pushAction("requested_peer");
 		push("button_id", data.buttonId);
 		push("peer_id", data.peerId.value);
+	}, [&](const ActionSetChatWallPaper &data) {
+		pushActor();
+		pushAction("set_chat_wallpaper");
+	}, [&](const ActionSetSameChatWallPaper &data) {
+		pushActor();
+		pushAction("set_same_chat_wallpaper");
+		pushReplyToMsgId("message_id");
 	}, [](v::null_t) {});
 
 	if (v::is_null(message.action.content)) {
