@@ -11,6 +11,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/platform/base_platform_info.h"
 #include "base/qt_signal_producer.h"
 #include "base/unixtime.h"
+#include "boxes/edit_caption_box.h"
 #include "chat_helpers/emoji_suggestions_widget.h"
 #include "chat_helpers/message_field.h"
 #include "menu/menu_send.h"
@@ -29,8 +30,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_user.h"
 #include "data/data_chat.h"
 #include "data/data_channel.h"
+#include "data/data_file_origin.h"
 #include "data/data_forum_topic.h"
 #include "data/data_peer_values.h"
+#include "data/data_photo_media.h"
 #include "data/stickers/data_stickers.h"
 #include "data/stickers/data_custom_emoji.h"
 #include "data/data_web_page.h"
@@ -39,6 +42,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_chat_participants.h"
 #include "ui/boxes/confirm_box.h"
 #include "ui/painter.h"
+#include "ui/power_saving.h"
 #include "history/history.h"
 #include "history/history_item.h"
 #include "history/view/controls/history_view_voice_record_bar.h"
@@ -800,7 +804,8 @@ void FieldHeader::paintEditOrReplyToMessage(Painter &p) {
 		.palette = &st::historyComposeAreaPalette,
 		.spoiler = Ui::Text::DefaultSpoilerCache(),
 		.now = crl::now(),
-		.paused = p.inactive(),
+		.pausedEmoji = p.inactive() || On(PowerSaving::kEmojiChat),
+		.pausedSpoiler = p.inactive() || On(PowerSaving::kChatSpoiler),
 		.elisionLines = 1,
 	});
 }
@@ -1020,9 +1025,6 @@ void ComposeControls::setCurrentDialogsEntryState(Dialogs::EntryState state) {
 	_currentDialogsEntryState = state;
 	updateForwarding();
 	registerDraftSource();
-	if (_inlineResults) {
-		_inlineResults->setCurrentDialogsEntryState(state);
-	}
 }
 
 PeerData *ComposeControls::sendAsPeer() const {
@@ -1067,7 +1069,7 @@ const HistoryItemsList &ComposeControls::forwardItems() const {
 }
 
 bool ComposeControls::focus() {
-	if (isRecording()) {
+	if (_wrap->isHidden() || _field->isHidden() || isRecording()) {
 		return false;
 	}
 	_field->setFocus();
@@ -1155,6 +1157,22 @@ void ComposeControls::setMimeDataHook(MimeDataHook hook) {
 	_field->setMimeDataHook(std::move(hook));
 }
 
+bool ComposeControls::confirmMediaEdit(Ui::PreparedList &list) {
+	if (!isEditingMessage()) {
+		return false;
+	} else if (_canReplaceMedia) {
+		EditCaptionBox::StartMediaReplace(
+			_window,
+			_editingId,
+			std::move(list),
+			_field->getTextWithTags(),
+			crl::guard(_wrap.get(), [=] { cancelEditMessage(); }));
+	} else {
+		_window->showToast({ tr::lng_edit_caption_attach(tr::now) });
+	}
+	return true;
+}
+
 rpl::producer<FileChosen> ComposeControls::fileChosen() const {
 	return _fileChosen.events();
 }
@@ -1429,23 +1447,17 @@ void ComposeControls::init() {
 		_voiceRecordBar->requestToSendWithOptions(options);
 	}, _wrap->lifetime());
 
-	{
-		const auto lastMsgId = _wrap->lifetime().make_state<FullMsgId>();
+	_header->editMsgId(
+	) | rpl::start_with_next([=](const auto &id) {
+		_editingId = id;
+	}, _wrap->lifetime());
 
-		_header->editMsgId(
-		) | rpl::filter([=](const auto &id) {
-			return !!id;
-		}) | rpl::start_with_next([=](const auto &id) {
-			*lastMsgId = id;
-		}, _wrap->lifetime());
-
-		session().data().itemRemoved(
-		) | rpl::filter([=](not_null<const HistoryItem*> item) {
-			return item->id && ((*lastMsgId) == item->fullId());
-		}) | rpl::start_with_next([=] {
-			cancelEditMessage();
-		}, _wrap->lifetime());
-	}
+	session().data().itemRemoved(
+	) | rpl::filter([=](not_null<const HistoryItem*> item) {
+		return (_editingId == item->fullId());
+	}) | rpl::start_with_next([=] {
+		cancelEditMessage();
+	}, _wrap->lifetime());
 
 	_window->materializeLocalDraftsRequests(
 	) | rpl::start_with_next([=] {
@@ -1908,22 +1920,29 @@ void ComposeControls::applyCloudDraft() {
 void ComposeControls::applyDraft(FieldHistoryAction fieldHistoryAction) {
 	Expects(_history != nullptr);
 
-	InvokeQueued(_autocomplete.get(), [=] { updateStickersByEmoji(); });
-	const auto guard = gsl::finally([&] {
-		updateSendButtonType();
-		updateControlsVisibility();
-		updateControlsGeometry(_wrap->size());
-	});
-
 	const auto editDraft = _history->draft(draftKey(DraftType::Edit));
 	const auto draft = editDraft
 		? editDraft
 		: _history->draft(draftKey(DraftType::Normal));
+	const auto editingId = (draft == editDraft)
+		? FullMsgId{ _history->peer->id, draft ? draft->msgId : 0 }
+		: FullMsgId();
+
+	InvokeQueued(_autocomplete.get(), [=] { updateStickersByEmoji(); });
+	const auto guard = gsl::finally([&] {
+		updateSendButtonType();
+		updateReplaceMediaButton();
+		updateControlsVisibility();
+		updateControlsGeometry(_wrap->size());
+	});
+
 	if (!draft) {
 		clearFieldText(0, fieldHistoryAction);
 		_field->setFocus();
 		_header->editMessage({});
 		_header->replyToMessage({});
+		_canReplaceMedia = false;
+		_photoEditMedia = nullptr;
 		return;
 	}
 
@@ -1937,9 +1956,11 @@ void ComposeControls::applyDraft(FieldHistoryAction fieldHistoryAction) {
 	}
 
 	if (draft == editDraft) {
-		_header->editMessage({ _history->peer->id, draft->msgId });
+		_header->editMessage(editingId);
 		_header->replyToMessage({});
 	} else {
+		_canReplaceMedia = false;
+		_photoEditMedia = nullptr;
 		_header->replyToMessage({ _history->peer->id, draft->msgId });
 		if (_header->replyingToMessage()) {
 			cancelForward();
@@ -2064,7 +2085,8 @@ void ComposeControls::initSendButton() {
 		_send.get(),
 		[=] { return sendButtonMenuType(); },
 		SendMenu::DefaultSilentCallback(send),
-		SendMenu::DefaultScheduleCallback(_wrap.get(), sendMenuType(), send));
+		SendMenu::DefaultScheduleCallback(_wrap.get(), sendMenuType(), send),
+		SendMenu::DefaultWhenOnlineCallback(send));
 }
 
 void ComposeControls::initSendAsButton(not_null<PeerData*> peer) {
@@ -2251,7 +2273,7 @@ void ComposeControls::finishAnimating() {
 }
 
 void ComposeControls::updateControlsGeometry(QSize size) {
-	// _attachToggle (_sendAs) -- _inlineResults ------ _tabbedPanel -- _fieldBarCancel
+	// (_attachToggle|_replaceMedia) (_sendAs) -- _inlineResults ------ _tabbedPanel -- _fieldBarCancel
 	// (_attachDocument|_attachPhoto) _field (_ttlInfo) (_silent|_botCommandStart) _tabbedSelectorToggle _send
 
 	const auto fieldWidth = size.width()
@@ -2276,6 +2298,9 @@ void ComposeControls::updateControlsGeometry(QSize size) {
 	const auto buttonsTop = size.height() - _attachToggle->height();
 
 	auto left = st::historySendRight;
+	if (_replaceMedia) {
+		_replaceMedia->moveToLeft(left, buttonsTop);
+	}
 	_attachToggle->moveToLeft(left, buttonsTop);
 	left += _attachToggle->width();
 	if (_sendAs) {
@@ -2322,6 +2347,12 @@ void ComposeControls::updateControlsVisibility() {
 	if (_sendAs) {
 		_sendAs->show();
 	}
+	if (_replaceMedia) {
+		_replaceMedia->show();
+		_attachToggle->hide();
+	} else {
+		_attachToggle->show();
+	}
 }
 
 bool ComposeControls::updateBotCommandShown() {
@@ -2403,6 +2434,7 @@ void ComposeControls::updateAttachBotsMenu() {
 	}
 	_attachBotsMenu = InlineBots::MakeAttachBotsMenu(
 		_parent,
+		_window,
 		_history->peer,
 		_sendActionFactory,
 		[=](bool compress) { _attachRequests.fire_copy(compress); });
@@ -2556,11 +2588,47 @@ void ComposeControls::editMessage(not_null<HistoryItem*> item) {
 			previewState));
 	applyDraft();
 
+	const auto media = item->media();
+	_canReplaceMedia = media && media->allowsEditMedia();
+	_photoEditMedia = (_canReplaceMedia
+		&& media->photo()
+		&& !media->photo()->isNull())
+		? media->photo()->createMediaView()
+		: nullptr;
+	if (_photoEditMedia) {
+		_photoEditMedia->wanted(Data::PhotoSize::Large, item->fullId());
+	}
+	if (updateReplaceMediaButton()) {
+		updateControlsVisibility();
+		updateControlsGeometry(_wrap->size());
+	}
+
 	if (_autocomplete) {
 		InvokeQueued(_autocomplete.get(), [=] { checkAutocomplete(); });
 	}
 }
 
+bool ComposeControls::updateReplaceMediaButton() {
+	if (!_canReplaceMedia) {
+		const auto result = (_replaceMedia != nullptr);
+		_replaceMedia = nullptr;
+		return result;
+	} else if (_replaceMedia) {
+		return false;
+	}
+	_replaceMedia = std::make_unique<Ui::IconButton>(
+		_wrap.get(),
+		st::historyReplaceMedia);
+	_replaceMedia->setClickedCallback([=] {
+		EditCaptionBox::StartMediaReplace(
+			_window,
+			_editingId,
+			_field->getTextWithTags(),
+			crl::guard(_wrap.get(), [=] { cancelEditMessage(); }));
+	});
+	return true;
+}
+
 void ComposeControls::cancelEditMessage() {
 	Expects(_history != nullptr);
 	Expects(draftKeyCurrent() != Data::DraftKey::None());
@@ -2858,8 +2926,6 @@ void ComposeControls::applyInlineBotQuery(
 			_inlineResults = std::make_unique<InlineBots::Layout::Widget>(
 				_parent,
 				_window);
-			_inlineResults->setCurrentDialogsEntryState(
-				_currentDialogsEntryState);
 			_inlineResults->setResultSelectedCallback([=](
 					InlineBots::ResultSelected result) {
 				if (result.open) {
