@@ -7,11 +7,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "history/view/history_view_spoiler_click_handler.h"
 
-#include "core/click_handler_types.h" // ClickHandlerContext
 #include "data/data_session.h"
 #include "history/view/history_view_element.h"
+#include "history/history.h"
 #include "main/main_session.h"
-#include "window/window_session_controller.h"
 #include "base/weak_ptr.h"
 
 namespace HistoryView {
@@ -22,18 +21,13 @@ void FillTextWithAnimatedSpoilers(
 	if (text.hasSpoilers()) {
 		text.setSpoilerLinkFilter([weak = base::make_weak(view)](
 				const ClickContext &context) {
-			const auto my = context.other.value<ClickHandlerContext>();
 			const auto button = context.button;
 			const auto view = weak.get();
-			if (button != Qt::LeftButton || !view || !my.elementDelegate) {
+			if (button != Qt::LeftButton || !view) {
 				return false;
-			} else if (const auto d = my.elementDelegate()) {
-				if (const auto controller = my.sessionWindow.get()) {
-					controller->session().data().registerShownSpoiler(view);
-				}
-				return true;
 			}
-			return false;
+			view->history()->owner().registerShownSpoiler(view);
+			return true;
 		});
 	}
 }
