@@ -21,6 +21,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/text/custom_emoji_instance.h"
 #include "ui/emoji_config.h"
 #include "ui/painter.h"
+#include "ui/power_saving.h"
 #include "core/application.h"
 #include "core/core_settings.h"
 #include "core/click_handler_types.h"
@@ -84,11 +85,7 @@ Sticker::Sticker(
 , _data(data)
 , _replacements(replacements)
 , _cachingTag(ChatHelpers::StickerLottieSize::MessageHistory)
-, _oncePlayed(false)
-, _premiumEffectPlayed(false)
-, _nextLastDiceFrame(false)
-, _skipPremiumEffect(skipPremiumEffect)
-, _giftBoxSticker(false) {
+, _skipPremiumEffect(skipPremiumEffect) {
 	if ((_dataMedia = _data->activeMediaView())) {
 		dataMediaCreated();
 	} else {
@@ -102,7 +99,14 @@ Sticker::Sticker(
 		if (_player) {
 			if (hasPremiumEffect() && !_premiumEffectPlayed) {
 				_premiumEffectPlayed = true;
-				_parent->delegate()->elementStartPremium(_parent, replacing);
+				if (On(PowerSaving::kStickersChat)
+					&& !_premiumEffectSkipped) {
+					_premiumEffectSkipped = true;
+				} else {
+					_parent->delegate()->elementStartPremium(
+						_parent,
+						replacing);
+				}
 			}
 			playerCreated();
 		}
@@ -236,6 +240,7 @@ DocumentData *Sticker::document() {
 void Sticker::stickerClearLoopPlayed() {
 	_oncePlayed = false;
 	_premiumEffectPlayed = false;
+	_premiumEffectSkipped = false;
 }
 
 void Sticker::paintAnimationFrame(
@@ -247,13 +252,17 @@ void Sticker::paintAnimationFrame(
 		: (context.selected() && !_nextLastDiceFrame)
 		? context.st->msgStickerOverlay()->c
 		: QColor(0, 0, 0, 0);
+	const auto powerSavingFlag = (isEmojiSticker() || _diceIndex >= 0)
+		? PowerSaving::kEmojiChat
+		: PowerSaving::kStickersChat;
+	const auto paused = context.paused || On(powerSavingFlag);
 	const auto frame = _player
 		? _player->frame(
 			_size,
 			colored,
 			mirrorHorizontal(),
 			context.now,
-			context.paused)
+			paused)
 		: StickerPlayer::FrameInfo();
 	if (_nextLastDiceFrame) {
 		_nextLastDiceFrame = false;
@@ -282,7 +291,7 @@ void Sticker::paintAnimationFrame(
 	const auto count = _player->framesCount();
 	_frameIndex = frame.index;
 	_framesCount = count;
-	_nextLastDiceFrame = !context.paused
+	_nextLastDiceFrame = !paused
 		&& (_diceIndex > 0)
 		&& (_frameIndex + 2 == count);
 	const auto playOnce = (_diceIndex > 0)
@@ -294,7 +303,7 @@ void Sticker::paintAnimationFrame(
 	const auto lastDiceFrame = (_diceIndex > 0) && atTheEnd();
 	const auto switchToNext = !playOnce
 		|| (!lastDiceFrame && (_frameIndex != 0 || !_oncePlayed));
-	if (!context.paused
+	if (!paused
 		&& switchToNext
 		&& _player->markFrameShown()
 		&& playOnce
@@ -458,6 +467,10 @@ void Sticker::emojiStickerClicked() {
 
 void Sticker::premiumStickerClicked() {
 	_premiumEffectPlayed = false;
+
+	// Remove when we start playing sticker itself on click.
+	_premiumEffectSkipped = false;
+
 	_parent->history()->owner().requestViewRepaint(_parent);
 }
 
@@ -523,7 +536,12 @@ void Sticker::setupPlayer() {
 void Sticker::checkPremiumEffectStart() {
 	if (!_premiumEffectPlayed && hasPremiumEffect()) {
 		_premiumEffectPlayed = true;
-		_parent->delegate()->elementStartPremium(_parent, nullptr);
+		if (On(PowerSaving::kStickersChat)
+			&& !_premiumEffectSkipped) {
+			_premiumEffectSkipped = true;
+		} else {
+			_parent->delegate()->elementStartPremium(_parent, nullptr);
+		}
 	}
 }
 
