@@ -945,6 +945,9 @@ AttachSelectorResult MakeJustSelectorMenu(
 	if (!AdjustMenuGeometryForSelector(menu, desiredPosition, selector)) {
 		return AttachSelectorResult::Failed;
 	}
+	if (mode != ChatHelpers::EmojiListMode::RecentReactions) {
+		Ui::Platform::FixPopupMenuNativeEmojiPopup(menu);
+	}
 	const auto selectorInnerTop = menu->preparedPadding().top()
 		- st::reactStripExtend.top();
 	menu->animatePhaseValue(
@@ -1006,6 +1009,7 @@ AttachSelectorResult AttachSelectorToMenu(
 	if (reactions.recent.empty() && !reactions.morePremiumAvailable) {
 		return AttachSelectorResult::Skipped;
 	}
+	const auto withSearch = reactions.customAllowed;
 	const auto selector = Ui::CreateChild<Selector>(
 		menu.get(),
 		controller,
@@ -1015,6 +1019,9 @@ AttachSelectorResult AttachSelectorToMenu(
 	if (!AdjustMenuGeometryForSelector(menu, desiredPosition, selector)) {
 		return AttachSelectorResult::Failed;
 	}
+	if (withSearch) {
+		Ui::Platform::FixPopupMenuNativeEmojiPopup(menu);
+	}
 	const auto selectorInnerTop = selector->useTransparency()
 		? (menu->preparedPadding().top() - st::reactStripExtend.top())
 		: st::lineWidth;
