@@ -37,6 +37,7 @@ constexpr auto kHoverScale = 1.24;
 	return std::make_shared<Ui::AnimatedIcon>(Ui::AnimatedIconDescriptor{
 		.generator = DocumentIconFrameGenerator(media),
 		.sizeOverride = QSize(size, size),
+		.colorized = media->owner()->emojiUsesTextColor(),
 	});
 }
 
@@ -51,6 +52,21 @@ Strip::Strip(
 , _inner(inner)
 , _finalSize(size)
 , _update(std::move(update)) {
+	style::PaletteChanged(
+	) | rpl::start_with_next([=] {
+		invalidateMainReactionImage();
+	}, _lifetime);
+}
+
+void Strip::invalidateMainReactionImage() {
+	if (_mainReactionImage.isNull()
+		&& !ranges::contains(_validEmoji, true)) {
+		return;
+	}
+	const auto was = base::take(_mainReactionMedia);
+	_mainReactionImage = QImage();
+	ranges::fill(_validEmoji, false);
+	resolveMainReactionIcon();
 }
 
 void Strip::applyList(
@@ -157,7 +173,11 @@ void Strip::paintOne(
 	} else {
 		const auto paintFrame = [&](not_null<Ui::AnimatedIcon*> animation) {
 			const auto size = int(std::floor(target.width() + 0.01));
-			const auto frame = animation->frame({ size, size }, _update);
+			const auto &textColor = st::windowFg->c;
+			const auto frame = animation->frame(
+				textColor,
+				{ size, size },
+				_update);
 			p.drawImage(target, frame.image);
 		};
 
@@ -218,9 +238,9 @@ int Strip::fillChosenIconGetIndex(ChosenReaction &chosen) const {
 	}
 	const auto &icon = *i;
 	if (const auto &appear = icon.appear; appear && appear->animating()) {
-		chosen.icon = appear->frame();
+		chosen.icon = appear->frame(st::windowFg->c);
 	} else if (const auto &select = icon.select; select && select->valid()) {
-		chosen.icon = select->frame();
+		chosen.icon = select->frame(st::windowFg->c);
 	}
 	return (i - begin(_icons));
 }
@@ -475,7 +495,7 @@ void Strip::setMainReactionIcon() {
 	if (i != end(_loadCache) && i->second.icon) {
 		const auto &icon = i->second.icon;
 		if (!icon->frameIndex() && icon->width() == MainReactionSize()) {
-			_mainReactionImage = i->second.icon->frame();
+			_mainReactionImage = i->second.icon->frame(st::windowFg->c);
 			return;
 		}
 	}
@@ -522,7 +542,8 @@ Ui::ImageSubrect Strip::validateEmoji(int frameIndex, float64 scale) {
 	p.fillRect(QRect(position, result.rect.size() / ratio), Qt::transparent);
 	if (_mainReactionImage.isNull()
 		&& _mainReactionIcon) {
-		_mainReactionImage = base::take(_mainReactionIcon)->frame();
+		_mainReactionImage = base::take(_mainReactionIcon)->frame(
+			st::windowFg->c);
 	}
 	if (!_mainReactionImage.isNull()) {
 		const auto target = QRect(
