@@ -89,7 +89,7 @@ Widget::Widget(
 		this,
 		account,
 		rpl::single(true))) {
-	Core::App().setDefaultFloatPlayerDelegate(floatPlayerDelegate());
+	controller->setDefaultFloatPlayerDelegate(floatPlayerDelegate());
 
 	getData()->country = ComputeNewAccountCountry();
 
@@ -175,6 +175,9 @@ not_null<Ui::RpWidget*> Widget::floatPlayerWidget() {
 	return this;
 }
 
+void Widget::floatPlayerToggleGifsPaused(bool paused) {
+}
+
 auto Widget::floatPlayerGetSection(Window::Column column)
 -> not_null<Media::Player::FloatSectionDelegate*> {
 	return this;
