@@ -364,14 +364,13 @@ void start(not_null<Core::Launcher*> launcher) {
 			// This value must come from TelegramForcePortable
 			// or from the "-workdir" command line argument.
 			cForceWorkingDir(cWorkingDir());
+			workingDirChosen = true;
 		} else {
-#if defined _DEBUG && !defined OS_MAC_STORE
-			cForceWorkingDir(cExeDir());
-#else // _DEBUG
+#if !defined _DEBUG || defined OS_MAC_STORE
 			cForceWorkingDir(psAppDataPath());
-#endif // !_DEBUG
+			workingDirChosen = true;
+#endif // !_DEBUG || OS_MAC_STORE
 		}
-		workingDirChosen = true;
 
 #if !defined Q_OS_MAC && !defined _DEBUG // fix first version
 		moveOldDataFrom = initialWorkingDir;
@@ -408,8 +407,6 @@ void start(not_null<Core::Launcher*> launcher) {
 		}
 	}
 
-	cForceWorkingDir(QDir(cWorkingDir()).absolutePath() + '/');
-
 // WinRT build requires the working dir to stay the same for plugin loading.
 #ifndef Q_OS_WINRT
 	QDir().setCurrent(cWorkingDir());
