@@ -52,10 +52,6 @@ constexpr auto kRememberShuffledOrderItems = 16;
 
 constexpr auto kMinLengthForSavePosition = 20 * TimeId(60); // 20 minutes.
 
-auto VoicePlaybackSpeed() {
-	return std::clamp(Core::App().settings().voicePlaybackSpeed(), 0.6, 1.7);
-}
-
 base::options::toggle OptionDisableAutoplayNext({
 	.id = kOptionDisableAutoplayNext,
 	.name = "Disable auto-play of the next track",
@@ -844,7 +840,7 @@ Streaming::PlaybackOptions Instance::streamingOptions(
 		? Streaming::Mode::Both
 		: Streaming::Mode::Audio;
 	result.speed = audioId.changeablePlaybackSpeed()
-		? VoicePlaybackSpeed()
+		? Core::App().settings().voicePlaybackSpeed()
 		: 1.;
 	result.audioId = audioId;
 	if (position >= 0) {
@@ -1140,7 +1136,8 @@ void Instance::updateVoicePlaybackSpeed() {
 			return;
 		}
 		if (const auto streamed = data->streamed.get()) {
-			streamed->instance.setSpeed(VoicePlaybackSpeed());
+			streamed->instance.setSpeed(
+				Core::App().settings().voicePlaybackSpeed());
 		}
 	}
 }
