@@ -14,6 +14,11 @@ class AudioMsgId;
 class DocumentData;
 class History;
 
+namespace Media {
+enum class RepeatMode;
+enum class OrderMode;
+} // namespace Media
+
 namespace Media {
 namespace Audio {
 class Instance;
@@ -45,18 +50,6 @@ namespace Player {
 
 extern const char kOptionDisableAutoplayNext[];
 
-enum class RepeatMode {
-	None,
-	One,
-	All,
-};
-
-enum class OrderMode {
-	Default,
-	Reverse,
-	Shuffle,
-};
-
 class Instance;
 struct TrackState;
 
