@@ -10,6 +10,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/gl/gl_shader.h"
 #include "ui/painter.h"
 #include "media/streaming/media_streaming_common.h"
+#include "platform/platform_overlay_widget.h"
 #include "base/platform/base_platform_info.h"
 #include "core/crash_reports.h"
 #include "styles/style_media_view.h"
@@ -27,7 +28,32 @@ constexpr auto kFooterOffset = kSaveMsgOffset + 4;
 constexpr auto kCaptionOffset = kFooterOffset + 4;
 constexpr auto kGroupThumbsOffset = kCaptionOffset + 4;
 constexpr auto kControlsOffset = kGroupThumbsOffset + 4;
-constexpr auto kControlValues = 2 * 4 + 4 * 4;
+constexpr auto kControlValues = 4 * 4 + 4 * 4; // over + icon
+
+[[nodiscard]] ShaderPart FragmentApplyControlsFade() {
+	return {
+		.header = R"(
+uniform sampler2D f_texture;
+uniform vec4 shadowTopRect;
+uniform vec2 shadowBottomAndOpacity;
+)",
+		.body = R"(
+	float topHeight = shadowTopRect.w;
+	float bottomHeight = shadowBottomAndOpacity.x;
+	float opacity = shadowBottomAndOpacity.y;
+	float viewportHeight = shadowTopRect.y + topHeight;
+	float fullHeight = topHeight + bottomHeight;
+	float topY = min(
+		(viewportHeight - gl_FragCoord.y) / fullHeight,
+		topHeight / fullHeight);
+	float topX = (gl_FragCoord.x - shadowTopRect.x) / shadowTopRect.z;
+	vec4 fadeTop = texture2D(f_texture, vec2(topX, topY)) * opacity;
+	float bottomY = max(fullHeight - gl_FragCoord.y, topHeight) / fullHeight;
+	vec4 fadeBottom = texture2D(f_texture, vec2(0.5, bottomY)) * opacity;
+	result.rgb = result.rgb * (1. - fadeTop.a) * (1. - fadeBottom.a);
+)",
+	};
+}
 
 [[nodiscard]] ShaderPart FragmentPlaceOnTransparentBackground() {
 	return {
@@ -46,12 +72,38 @@ uniform float transparentSize;
 	};
 }
 
+[[nodiscard]] ShaderPart FragmentRoundedCorners() {
+	return {
+		.header = R"(
+uniform vec4 roundRect;
+uniform float roundRadius;
+
+float roundedCorner() {
+	vec2 rectHalf = roundRect.zw / 2.;
+	vec2 rectCenter = roundRect.xy + rectHalf;
+	vec2 fromRectCenter = abs(gl_FragCoord.xy - rectCenter);
+	vec2 vectorRadius = vec2(roundRadius + 0.5, roundRadius + 0.5);
+	vec2 fromCenterWithRadius = fromRectCenter + vectorRadius;
+	vec2 fromRoundingCenter = max(fromCenterWithRadius, rectHalf)
+		- rectHalf;
+	float rounded = length(fromRoundingCenter) - roundRadius;
+
+	return 1. - smoothstep(0., 1., rounded);
+}
+)",
+		.body = R"(
+	result = vec4(roundedCorner());
+)",
+	};
+}
+
 } // namespace
 
 OverlayWidget::RendererGL::RendererGL(not_null<OverlayWidget*> owner)
 : _owner(owner) {
 	style::PaletteChanged(
 	) | rpl::start_with_next([=] {
+		_controlsFadeImage.invalidate();
 		_radialImage.invalidate();
 		_documentBubbleImage.invalidate();
 		_themePreviewImage.invalidate();
@@ -69,7 +121,10 @@ void OverlayWidget::RendererGL::init(
 	constexpr auto kQuadVertices = kQuads * 4;
 	constexpr auto kQuadValues = kQuadVertices * 4;
 	constexpr auto kControlsValues = kControlsCount * kControlValues;
-	constexpr auto kValues = kQuadValues + kControlsValues;
+	constexpr auto kRoundingQuads = 4;
+	constexpr auto kRoundingVertices = kRoundingQuads * 6;
+	constexpr auto kRoundingValues = kRoundingVertices * 2;
+	constexpr auto kValues = kQuadValues + kControlsValues + kRoundingValues;
 
 	_contentBuffer.emplace();
 	_contentBuffer->setUsagePattern(QOpenGLBuffer::DynamicDraw);
@@ -90,6 +145,15 @@ void OverlayWidget::RendererGL::init(
 			FragmentSampleARGB32Texture(),
 		})).vertex;
 
+	_staticContentProgram.emplace();
+	LinkProgram(
+		&*_staticContentProgram,
+		_texturedVertexShader,
+		FragmentShader({
+			FragmentSampleARGB32Texture(),
+			FragmentApplyControlsFade()
+		}));
+
 	_withTransparencyProgram.emplace();
 	LinkProgram(
 		&*_withTransparencyProgram,
@@ -97,6 +161,7 @@ void OverlayWidget::RendererGL::init(
 		FragmentShader({
 			FragmentSampleARGB32Texture(),
 			FragmentPlaceOnTransparentBackground(),
+			FragmentApplyControlsFade()
 		}));
 
 	_yuv420Program.emplace();
@@ -105,6 +170,7 @@ void OverlayWidget::RendererGL::init(
 		_texturedVertexShader,
 		FragmentShader({
 			FragmentSampleYUV420Texture(),
+			FragmentApplyControlsFade()
 		}));
 
 	_nv12Program.emplace();
@@ -113,6 +179,7 @@ void OverlayWidget::RendererGL::init(
 		_texturedVertexShader,
 		FragmentShader({
 			FragmentSampleNV12Texture(),
+			FragmentApplyControlsFade()
 		}));
 
 	_fillProgram.emplace();
@@ -130,6 +197,12 @@ void OverlayWidget::RendererGL::init(
 			FragmentGlobalOpacity(),
 		}));
 
+	_roundedCornersProgram.emplace();
+	LinkProgram(
+		&*_roundedCornersProgram,
+		VertexShader({ VertexViewportTransform() }),
+		FragmentShader({ FragmentRoundedCorners() }));
+
 	const auto renderer = reinterpret_cast<const char*>(
 		f.glGetString(GL_RENDERER));
 	CrashReports::SetAnnotation(
@@ -161,6 +234,12 @@ void OverlayWidget::RendererGL::paint(
 	if (_factor != factor) {
 		_factor = factor;
 		_controlsImage.invalidate();
+
+		// We use the fact that fade texture atlas
+		// takes exactly full texture size. In case we
+		// just invalidate it we may get larger image
+		// in case of moving from greater _factor to lesser.
+		_controlsFadeImage.destroy(&f);
 	}
 	_blendingEnabled = false;
 	_viewport = widget->size();
@@ -249,7 +328,12 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 		}
 		_chromaNV12 = nv12;
 	}
-	if (!nv12) {
+
+	validateControlsFade();
+	if (nv12) {
+		_f->glActiveTexture(GL_TEXTURE2);
+		_controlsFadeImage.bind(*_f);
+	} else {
 		_f->glActiveTexture(GL_TEXTURE2);
 		_textures.bind(*_f, 3);
 		if (upload) {
@@ -263,6 +347,9 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 			_chromaSize = yuv->chromaSize;
 			_f->glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
 		}
+
+		_f->glActiveTexture(GL_TEXTURE3);
+		_controlsFadeImage.bind(*_f);
 	}
 	program->setUniformValue("y_texture", GLint(0));
 	if (nv12) {
@@ -271,6 +358,7 @@ void OverlayWidget::RendererGL::paintTransformedVideoFrame(
 		program->setUniformValue("u_texture", GLint(1));
 		program->setUniformValue("v_texture", GLint(2));
 	}
+	program->setUniformValue("f_texture", GLint(nv12 ? 2 : 3));
 
 	toggleBlending(false);
 	paintTransformedContent(program, geometry);
@@ -291,7 +379,7 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 
 	auto &program = fillTransparentBackground
 		? _withTransparencyProgram
-		: _imageProgram;
+		: _staticContentProgram;
 	program->bind();
 	if (fillTransparentBackground) {
 		program->setUniformValue(
@@ -335,7 +423,13 @@ void OverlayWidget::RendererGL::paintTransformedStaticContent(
 			_rgbaSize = image.size();
 		}
 	}
+
+	validateControlsFade();
+	_f->glActiveTexture(GL_TEXTURE1);
+	_controlsFadeImage.bind(*_f);
+
 	program->setUniformValue("s_texture", GLint(0));
+	program->setUniformValue("f_texture", GLint(1));
 
 	toggleBlending(semiTransparent && !fillTransparentBackground);
 	paintTransformedContent(&*program, geometry);
@@ -378,6 +472,15 @@ void OverlayWidget::RendererGL::paintTransformedContent(
 	_contentBuffer->write(0, coords, sizeof(coords));
 
 	program->setUniformValue("viewport", _uniformViewport);
+	const auto &top = st::mediaviewShadowTop.size();
+	const auto point = QPoint(_shadowTopFlip ? 0 : (_viewport.width() - top.width()), 0);
+	program->setUniformValue(
+		"shadowTopRect",
+		Uniform(transformRect(QRect(point, top))));
+	const auto &bottom = st::mediaviewShadowBottom;
+	program->setUniformValue(
+		"shadowBottomAndOpacity",
+		QVector2D(bottom.height() * _factor, geometry.controlsOpacity));
 
 	FillTexturedRectangle(*_f, &*program);
 }
@@ -428,8 +531,8 @@ void OverlayWidget::RendererGL::paintRadialLoading(
 
 void OverlayWidget::RendererGL::paintThemePreview(QRect outer) {
 	paintUsingRaster(_themePreviewImage, outer, [&](Painter &&p) {
-		const auto newOuter = QRect(QPoint(), outer.size());
-		_owner->paintThemePreviewContent(p, newOuter, newOuter);
+		p.translate(-outer.topLeft());
+		_owner->paintThemePreviewContent(p, outer, outer);
 	}, kThemePreviewOffset);
 }
 
@@ -460,28 +563,37 @@ void OverlayWidget::RendererGL::paintControlsStart() {
 
 void OverlayWidget::RendererGL::paintControl(
 		OverState control,
-		QRect outer,
-		float64 outerOpacity,
+		QRect over,
+		float64 overOpacity,
 		QRect inner,
 		float64 innerOpacity,
 		const style::icon &icon) {
 	const auto meta = ControlMeta(control);
 	Assert(meta.icon == &icon);
 
-	const auto &bg = st::mediaviewControlBg->c;
-	const auto bgAlpha = int(base::SafeRound(bg.alpha() * outerOpacity));
+	const auto overAlpha = overOpacity * kOverBackgroundOpacity;
 	const auto offset = kControlsOffset + (meta.index * kControlValues) / 4;
-	const auto fgOffset = offset + 2;
-	const auto bgRect = transformRect(outer);
+	const auto fgOffset = offset + 4;
+	const auto overRect = _controlsImage.texturedRect(
+		over,
+		_controlsTextures[kControlsCount]);
+	const auto overGeometry = transformRect(over);
 	const auto iconRect = _controlsImage.texturedRect(
 		inner,
 		_controlsTextures[meta.index]);
 	const auto iconGeometry = transformRect(iconRect.geometry);
 	const GLfloat coords[] = {
-		bgRect.left(), bgRect.top(),
-		bgRect.right(), bgRect.top(),
-		bgRect.right(), bgRect.bottom(),
-		bgRect.left(), bgRect.bottom(),
+		overGeometry.left(), overGeometry.top(),
+		overRect.texture.left(), overRect.texture.bottom(),
+
+		overGeometry.right(), overGeometry.top(),
+		overRect.texture.right(), overRect.texture.bottom(),
+
+		overGeometry.right(), overGeometry.bottom(),
+		overRect.texture.right(), overRect.texture.top(),
+
+		overGeometry.left(), overGeometry.bottom(),
+		overRect.texture.left(), overRect.texture.top(),
 
 		iconGeometry.left(), iconGeometry.top(),
 		iconRect.texture.left(), iconRect.texture.bottom(),
@@ -495,27 +607,22 @@ void OverlayWidget::RendererGL::paintControl(
 		iconGeometry.left(), iconGeometry.bottom(),
 		iconRect.texture.left(), iconRect.texture.top(),
 	};
-	if (!outer.isEmpty() && bgAlpha > 0) {
+	_controlsProgram->bind();
+	_controlsProgram->setUniformValue("viewport", _uniformViewport);
+	if (!over.isEmpty() && overOpacity > 0) {
 		_contentBuffer->write(
 			offset * 4 * sizeof(GLfloat),
 			coords,
 			sizeof(coords));
-		_fillProgram->bind();
-		_fillProgram->setUniformValue("viewport", _uniformViewport);
-		FillRectangle(
-			*_f,
-			&*_fillProgram,
-			offset,
-			QColor(bg.red(), bg.green(), bg.blue(), bgAlpha));
+		_controlsProgram->setUniformValue("g_opacity", GLfloat(overAlpha));
+		FillTexturedRectangle(*_f, &*_controlsProgram, offset);
 	} else {
 		_contentBuffer->write(
 			fgOffset * 4 * sizeof(GLfloat),
 			coords + (fgOffset - offset) * 4,
 			sizeof(coords) - (fgOffset - offset) * 4 * sizeof(GLfloat));
 	}
-	_controlsProgram->bind();
 	_controlsProgram->setUniformValue("g_opacity", GLfloat(innerOpacity));
-	_controlsProgram->setUniformValue("viewport", _uniformViewport);
 	FillTexturedRectangle(*_f, &*_controlsProgram, fgOffset);
 }
 
@@ -524,10 +631,9 @@ auto OverlayWidget::RendererGL::ControlMeta(OverState control)
 	switch (control) {
 	case OverLeftNav: return { 0, &st::mediaviewLeft };
 	case OverRightNav: return { 1, &st::mediaviewRight };
-	case OverClose: return { 2, &st::mediaviewClose };
-	case OverSave: return { 3, &st::mediaviewSave };
-	case OverRotate: return { 4, &st::mediaviewRotate };
-	case OverMore: return { 5, &st::mediaviewMore };
+	case OverSave: return { 2, &st::mediaviewSave };
+	case OverRotate: return { 3, &st::mediaviewRotate };
+	case OverMore: return { 4, &st::mediaviewMore };
 	}
 	Unexpected("Control value in OverlayWidget::RendererGL::ControlIndex.");
 }
@@ -539,7 +645,6 @@ void OverlayWidget::RendererGL::validateControls() {
 	const auto metas = {
 		ControlMeta(OverLeftNav),
 		ControlMeta(OverRightNav),
-		ControlMeta(OverClose),
 		ControlMeta(OverSave),
 		ControlMeta(OverRotate),
 		ControlMeta(OverMore),
@@ -550,6 +655,8 @@ void OverlayWidget::RendererGL::validateControls() {
 		maxWidth = std::max(meta.icon->width(), maxWidth);
 		fullHeight += meta.icon->height();
 	}
+	maxWidth = std::max(st::mediaviewIconOver, maxWidth);
+	fullHeight += st::mediaviewIconOver;
 	auto image = QImage(
 		QSize(maxWidth, fullHeight) * _factor,
 		QImage::Format_ARGB32_Premultiplied);
@@ -566,6 +673,15 @@ void OverlayWidget::RendererGL::validateControls() {
 				meta.icon->size() * _factor);
 			height += meta.icon->height();
 		}
+		auto hq = PainterHighQualityEnabler(p);
+		p.setPen(Qt::NoPen);
+		p.setBrush(OverBackgroundColor());
+		p.drawEllipse(
+			QRect(0, height, st::mediaviewIconOver, st::mediaviewIconOver));
+		_controlsTextures[index++] = QRect(
+			QPoint(0, height) * _factor,
+			QSize(st::mediaviewIconOver, st::mediaviewIconOver) * _factor);
+		height += st::mediaviewIconOver;
 	}
 	_controlsImage.setImage(std::move(image));
 }
@@ -575,6 +691,43 @@ void OverlayWidget::RendererGL::invalidateControls() {
 	ranges::fill(_controlsTextures, QRect());
 }
 
+void OverlayWidget::RendererGL::validateControlsFade() {
+	const auto flip = !_owner->topShadowOnTheRight();
+	if (!_controlsFadeImage.image().isNull()
+		&& _shadowTopFlip == flip) {
+		return;
+	}
+	_shadowTopFlip = flip;
+	const auto width = st::mediaviewShadowTop.width();
+	const auto bottomTop = st::mediaviewShadowTop.height();
+	const auto height = bottomTop + st::mediaviewShadowBottom.height();
+
+	auto image = QImage(
+		QSize(width, height) * _factor,
+		QImage::Format_ARGB32_Premultiplied);
+	image.fill(Qt::transparent);
+	image.setDevicePixelRatio(_factor);
+
+	auto p = QPainter(&image);
+	st::mediaviewShadowTop.paint(p, 0, 0, width);
+	st::mediaviewShadowBottom.fill(
+		p,
+		QRect(0, bottomTop, width, st::mediaviewShadowBottom.height()));
+	p.end();
+
+	if (flip) {
+		image = std::move(image).mirrored(true, false);
+	}
+
+	_controlsFadeImage.setImage(std::move(image));
+	_shadowTopTexture = QRect(
+		QPoint(),
+		QSize(width, st::mediaviewShadowTop.height()) * _factor);
+	_shadowBottomTexture = QRect(
+		QPoint(0, bottomTop) * _factor,
+		QSize(width, st::mediaviewShadowBottom.height()) * _factor);
+}
+
 void OverlayWidget::RendererGL::paintFooter(QRect outer, float64 opacity) {
 	paintUsingRaster(_footerImage, outer, [&](Painter &&p) {
 		const auto newOuter = QRect(QPoint(), outer.size());
@@ -598,24 +751,94 @@ void OverlayWidget::RendererGL::paintGroupThumbs(
 	}, kGroupThumbsOffset, true);
 }
 
-void OverlayWidget::RendererGL::invalidate() {
-	_trackFrameIndex = -1;
-	_streamedIndex = -1;
-	const auto images = {
-		&_radialImage,
-		&_documentBubbleImage,
-		&_themePreviewImage,
-		&_saveMsgImage,
-		&_footerImage,
-		&_captionImage,
-		&_groupThumbsImage,
-		&_controlsImage,
+void OverlayWidget::RendererGL::paintRoundedCorners(int radius) {
+	const auto topLeft = transformRect(QRect(0, 0, radius, radius));
+	const auto topRight = transformRect(
+		QRect(_viewport.width() - radius, 0, radius, radius));
+	const auto bottomRight = transformRect(QRect(
+		_viewport.width() - radius,
+		_viewport.height() - radius,
+		radius,
+		radius));
+	const auto bottomLeft = transformRect(
+		QRect(0, _viewport.height() - radius, radius, radius));
+	const GLfloat coords[] = {
+		topLeft.left(), topLeft.top(),
+		topLeft.right(), topLeft.top(),
+		topLeft.right(), topLeft.bottom(),
+		topLeft.right(), topLeft.bottom(),
+		topLeft.left(), topLeft.bottom(),
+		topLeft.left(), topLeft.top(),
+
+		topRight.left(), topRight.top(),
+		topRight.right(), topRight.top(),
+		topRight.right(), topRight.bottom(),
+		topRight.right(), topRight.bottom(),
+		topRight.left(), topRight.bottom(),
+		topRight.left(), topRight.top(),
+
+		bottomRight.left(), bottomRight.top(),
+		bottomRight.right(), bottomRight.top(),
+		bottomRight.right(), bottomRight.bottom(),
+		bottomRight.right(), bottomRight.bottom(),
+		bottomRight.left(), bottomRight.bottom(),
+		bottomRight.left(), bottomRight.top(),
+
+		bottomLeft.left(), bottomLeft.top(),
+		bottomLeft.right(), bottomLeft.top(),
+		bottomLeft.right(), bottomLeft.bottom(),
+		bottomLeft.right(), bottomLeft.bottom(),
+		bottomLeft.left(), bottomLeft.bottom(),
+		bottomLeft.left(), bottomLeft.top(),
 	};
-	for (const auto image : images) {
-		image->setImage(QImage());
-	}
-	invalidateControls();
+	const auto offset = kControlsOffset
+		+ (kControlsCount * kControlValues) / 4;
+	const auto byteOffset = offset * 4 * sizeof(GLfloat);
+	_contentBuffer->write(byteOffset, coords, sizeof(coords));
+	_roundedCornersProgram->bind();
+	_roundedCornersProgram->setUniformValue("viewport", _uniformViewport);
+	const auto roundRect = transformRect(QRect(QPoint(), _viewport));
+	_roundedCornersProgram->setUniformValue("roundRect", Uniform(roundRect));
+	_roundedCornersProgram->setUniformValue(
+		"roundRadius",
+		GLfloat(radius * _factor));
+
+	_f->glEnable(GL_BLEND);
+	_f->glBlendFunc(GL_ZERO, GL_SRC_ALPHA);
+
+	GLint position = _roundedCornersProgram->attributeLocation("position");
+	_f->glVertexAttribPointer(
+		position,
+		2,
+		GL_FLOAT,
+		GL_FALSE,
+		2 * sizeof(GLfloat),
+		reinterpret_cast<const void*>(byteOffset));
+	_f->glEnableVertexAttribArray(position);
+
+	_f->glDrawArrays(GL_TRIANGLES, 0, base::array_size(coords) / 2);
+
+	_f->glDisableVertexAttribArray(position);
 }
+//
+//void OverlayWidget::RendererGL::invalidate() {
+//	_trackFrameIndex = -1;
+//	_streamedIndex = -1;
+//	const auto images = {
+//		&_radialImage,
+//		&_documentBubbleImage,
+//		&_themePreviewImage,
+//		&_saveMsgImage,
+//		&_footerImage,
+//		&_captionImage,
+//		&_groupThumbsImage,
+//		&_controlsImage,
+//	};
+//	for (const auto image : images) {
+//		image->setImage(QImage());
+//	}
+//	invalidateControls();
+//}
 
 void OverlayWidget::RendererGL::paintUsingRaster(
 		Ui::GL::Image &image,
@@ -627,6 +850,7 @@ void OverlayWidget::RendererGL::paintUsingRaster(
 	const auto size = rect.size() * _factor;
 	if (raster.width() < size.width() || raster.height() < size.height()) {
 		raster = QImage(size, QImage::Format_ARGB32_Premultiplied);
+		Assert(!raster.isNull());
 		raster.setDevicePixelRatio(_factor);
 		if (!transparent
 			&& (raster.width() > size.width()
