@@ -9,6 +9,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "ui/painter.h"
 #include "media/view/media_view_pip.h"
+#include "platform/platform_overlay_widget.h"
+#include "styles/style_media_view.h"
 
 namespace Media::View {
 
@@ -73,6 +75,7 @@ void OverlayWidget::RendererSW::paintTransformedVideoFrame(
 		return;
 	}
 	paintTransformedImage(_owner->videoFrame(), rect, rotation);
+	paintControlsFade(rect, geometry.controlsOpacity);
 }
 
 void OverlayWidget::RendererSW::paintTransformedStaticContent(
@@ -89,10 +92,45 @@ void OverlayWidget::RendererSW::paintTransformedStaticContent(
 	if (fillTransparentBackground) {
 		_p->fillRect(rect, _transparentBrush);
 	}
-	if (image.isNull()) {
-		return;
+	if (!image.isNull()) {
+		paintTransformedImage(image, rect, rotation);
 	}
-	paintTransformedImage(image, rect, rotation);
+	paintControlsFade(rect, geometry.controlsOpacity);
+}
+
+void OverlayWidget::RendererSW::paintControlsFade(
+		QRect geometry,
+		float64 opacity) {
+	_p->setOpacity(opacity);
+	_p->setClipRect(geometry);
+	const auto width = _owner->width();
+	const auto &top = st::mediaviewShadowTop;
+	const auto flip = !_owner->topShadowOnTheRight();
+	const auto topShadow = QRect(
+		QPoint(flip ? 0 : (width - top.width()), 0),
+		top.size());
+	if (topShadow.intersected(geometry).intersects(_clipOuter)) {
+		if (flip) {
+			if (_topShadowCache.isNull()
+				|| _topShadowColor != st::windowShadowFg->c) {
+				_topShadowColor = st::windowShadowFg->c;
+				_topShadowCache = top.instance(
+					_topShadowColor).mirrored(true, false);
+			}
+			_p->drawImage(0, 0, _topShadowCache);
+		} else {
+			top.paint(*_p, topShadow.topLeft(), width);
+		}
+	}
+	const auto &bottom = st::mediaviewShadowBottom;
+	const auto bottomShadow = QRect(
+		QPoint(0, _owner->height() - bottom.height()),
+		QSize(width, bottom.height()));
+	if (bottomShadow.intersected(geometry).intersects(_clipOuter)) {
+		bottom.fill(*_p, bottomShadow);
+	}
+	_p->setClipping(false);
+	_p->setOpacity(1.);
 }
 
 void OverlayWidget::RendererSW::paintTransformedImage(
@@ -147,22 +185,20 @@ void OverlayWidget::RendererSW::paintControlsStart() {
 
 void OverlayWidget::RendererSW::paintControl(
 		OverState control,
-		QRect outer,
-		float64 outerOpacity,
+		QRect over,
+		float64 overOpacity,
 		QRect inner,
 		float64 innerOpacity,
 		const style::icon &icon) {
-	if (!outer.isEmpty() && !outer.intersects(_clipOuter)) {
+	if (!over.isEmpty() && !over.intersects(_clipOuter)) {
 		return;
 	}
-	if (!outer.isEmpty() && outerOpacity > 0) {
-		_p->setOpacity(outerOpacity);
-		for (const auto &rect : *_clip) {
-			const auto fill = outer.intersected(rect);
-			if (!fill.isEmpty()) {
-				_p->fillRect(fill, st::mediaviewControlBg);
-			}
+	if (!over.isEmpty() && overOpacity > 0) {
+		if (_overControlImage.isNull()) {
+			validateOverControlImage();
 		}
+		_p->setOpacity(overOpacity);
+		_p->drawImage(over.topLeft(), _overControlImage);
 	}
 	if (inner.intersects(_clipOuter)) {
 		_p->setOpacity(innerOpacity);
@@ -194,4 +230,22 @@ void OverlayWidget::RendererSW::paintRoundedCorners(int radius) {
 	// The RpWindow rounding overlay will do the job.
 }
 
+void OverlayWidget::RendererSW::validateOverControlImage() {
+	const auto size = QSize(st::mediaviewIconOver, st::mediaviewIconOver);
+	const auto alpha = base::SafeRound(kOverBackgroundOpacity * 255);
+	_overControlImage = QImage(
+		size * style::DevicePixelRatio(),
+		QImage::Format_ARGB32_Premultiplied);
+	_overControlImage.setDevicePixelRatio(style::DevicePixelRatio());
+	_overControlImage.fill(Qt::transparent);
+
+	Painter p(&_overControlImage);
+	PainterHighQualityEnabler hq(p);
+	p.setPen(Qt::NoPen);
+	auto color = OverBackgroundColor();
+	color.setAlpha(alpha);
+	p.setBrush(color);
+	p.drawEllipse(QRect(QPoint(), size));
+}
+
 } // namespace Media::View
