@@ -35,6 +35,11 @@ constexpr ShiftedDcId updaterDcId(DcId dcId) {
 	return ShiftDcId(dcId, kUpdaterDcShift);
 }
 
+// send(MTPupload_GetFile(), MTP::groupCallStreamDcId(dc)) - for group call stream
+constexpr ShiftedDcId groupCallStreamDcId(DcId dcId) {
+	return ShiftDcId(dcId, kGroupCallStreamDcShift);
+}
+
 constexpr auto kUploadSessionsCount = 2;
 
 namespace details {
@@ -57,6 +62,14 @@ inline constexpr bool isDownloadDcId(ShiftedDcId shiftedDcId) {
 		&& (shiftedDcId < details::downloadDcId(0, kMaxMediaDcCount - 1) + kDcShift);
 }
 
+inline constexpr bool isMediaClusterDcId(ShiftedDcId shiftedDcId) {
+	const auto shift = GetDcIdShift(shiftedDcId);
+	return isDownloadDcId(shiftedDcId)
+		|| (shift == kGroupCallStreamDcShift)
+		|| (shift == kExportMediaDcShift)
+		|| (shift == kUpdaterDcShift);
+}
+
 inline bool isCdnDc(MTPDdcOption::Flags flags) {
 	return (flags & MTPDdcOption::Flag::f_cdn);
 }
