@@ -24,15 +24,11 @@ struct ConfigFields {
 	int onlineCloudTimeout = 300000;
 	int notifyCloudDelay = 30000;
 	int notifyDefaultDelay = 1500;
-	int savedGifsLimit = 200;
 	int editTimeLimit = 172800;
 	int revokeTimeLimit = 172800;
 	int revokePrivateTimeLimit = 172800;
 	bool revokePrivateInbox = false;
 	int stickersRecentLimit = 30;
-	int stickersFavedLimit = 5;
-	rpl::variable<int> pinnedDialogsCountMax = 5;
-	rpl::variable<int> pinnedDialogsInFolderMax = 100;
 	QString internalLinksDomain = u"https://t.me/"_q;
 	int channelsReadMediaPeriod = 86400 * 7;
 	int callReceiveTimeoutMs = 20000;
@@ -45,6 +41,7 @@ struct ConfigFields {
 	int captionLengthMax = 1024;
 	QString reactionDefaultEmoji = ConfigDefaultReactionEmoji();
 	uint64 reactionDefaultCustom;
+	QString autologinToken;
 };
 
 class Config final {
@@ -79,9 +76,7 @@ public:
 
 	// Set from legacy local stored values.
 	void setChatSizeMax(int value);
-	void setSavedGifsLimit(int value);
 	void setStickersRecentLimit(int value);
-	void setStickersFavedLimit(int value);
 	void setMegagroupSizeMax(int value);
 	void setTxtDomainString(const QString &value);
 
