@@ -30,8 +30,6 @@ public:
 
 	void updateWindowIcon() override;
 
-	bool preventsQuit(Core::QuitReason reason) override;
-
 	class Private;
 
 protected:
@@ -87,4 +85,8 @@ private:
 
 };
 
+[[nodiscard]] inline int32 ScreenNameChecksum(const QString &name) {
+	return Window::DefaultScreenNameChecksum(name);
+}
+
 } // namespace Platform
