@@ -21,23 +21,30 @@ namespace Settings {
 
 void SetupConnectionType(
 	not_null<Window::Controller*> controller,
-	not_null<Main::Account*> account,
+	not_null<::Main::Account*> account,
 	not_null<Ui::VerticalLayout*> container);
 bool HasUpdate();
 void SetupUpdate(
 	not_null<Ui::VerticalLayout*> container,
 	Fn<void(Type)> showOther);
+void SetupWindowTitleContent(
+	Window::SessionController *controller,
+	not_null<Ui::VerticalLayout*> container);
 void SetupSystemIntegrationContent(
 	Window::SessionController *controller,
 	not_null<Ui::VerticalLayout*> container);
-void SetupAnimations(not_null<Ui::VerticalLayout*> container);
+void SetupAnimations(
+	not_null<Window::Controller*> window,
+	not_null<Ui::VerticalLayout*> container);
 
-class Advanced : public Section {
+class Advanced : public Section<Advanced> {
 public:
 	Advanced(
 		QWidget *parent,
 		not_null<Window::SessionController*> controller);
 
+	[[nodiscard]] rpl::producer<QString> title() override;
+
 	rpl::producer<Type> sectionShowOther() override;
 
 private:
