@@ -28,6 +28,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/notifications_manager.h"
 #include "settings/settings_common.h"
 #include "storage/localimageloader.h"
+#include "data/data_document_resolver.h"
 #include "styles/style_settings.h"
 #include "styles/style_layers.h"
 
@@ -148,6 +149,7 @@ void SetupExperimental(
 	addToggle(kOptionAutoScrollInactiveChat);
 	addToggle(Window::Notifications::kOptionGNotification);
 	addToggle(Core::kOptionFreeType);
+	addToggle(Data::kOptionExternalVideoPlayer);
 }
 
 } // namespace
