@@ -188,7 +188,7 @@ public:
 		not_null<Ui::VerticalLayout*> container,
 		not_null<Window::SessionController*> controller);
 
-	[[nodiscard]] rpl::producer<> currentAccountActivations() const;
+	[[nodiscard]] rpl::producer<> closeRequests() const;
 
 private:
 	void setup();
@@ -209,7 +209,7 @@ private:
 	std::unique_ptr<Ui::VerticalLayoutReorder> _reorder;
 	int _reordering = 0;
 
-	rpl::event_stream<> _currentAccountActivations;
+	rpl::event_stream<> _closeRequests;
 
 	base::binary_guard _accountSwitchGuard;
 
@@ -419,7 +419,8 @@ void SetupRows(
 		std::move(value),
 		tr::lng_context_copy_mention(tr::now),
 		[=] {
-			const auto box = controller->show(Box(UsernamesBox, session));
+			const auto box = controller->show(
+				Box(UsernamesBox, session->user()));
 			box->boxClosing(
 			) | rpl::start_with_next([=] {
 				session->api().usernames().requestToCache(session->user());
@@ -676,8 +677,7 @@ void SetupAccountsWrap(
 			state->menu);
 		addAction(tr::lng_context_new_window(tr::now), [=] {
 			Ui::PreventDelayedActivation();
-			Core::App().ensureSeparateWindowForAccount(account);
-			Core::App().domain().maybeActivate(account);
+			callback(Qt::ControlModifier);
 		}, &st::menuIconNewWindow);
 		Window::AddSeparatorAndShiftUp(addAction);
 
@@ -728,8 +728,8 @@ AccountsList::AccountsList(
 	setup();
 }
 
-rpl::producer<> AccountsList::currentAccountActivations() const {
-	return _currentAccountActivations.events();
+rpl::producer<> AccountsList::closeRequests() const {
+	return _closeRequests.events();
 }
 
 void AccountsList::setup() {
@@ -887,7 +887,7 @@ void AccountsList::rebuild() {
 					return;
 				}
 				if (account == &_controller->session().account()) {
-					_currentAccountActivations.fire({});
+					_closeRequests.fire({});
 					return;
 				}
 				const auto newWindow = (modifiers & Qt::ControlModifier);
@@ -895,14 +895,16 @@ void AccountsList::rebuild() {
 					if (guard) {
 						_reorder->finishReordering();
 						if (newWindow) {
+							_closeRequests.fire({});
 							Core::App().ensureSeparateWindowForAccount(
 								account);
 						}
 						Core::App().domain().maybeActivate(account);
 					}
 				};
-				if (Core::App().separateWindowForAccount(account)) {
-					activate();
+				if (const auto window = Core::App().separateWindowForAccount(account)) {
+					_closeRequests.fire({});
+					window->activate();
 				} else {
 					base::call_delayed(
 						st::defaultRippleAnimation.hideDuration,
@@ -966,7 +968,7 @@ AccountsEvents SetupAccounts(
 		container,
 		controller);
 	return {
-		.currentAccountActivations = list->currentAccountActivations(),
+		.closeRequests = list->closeRequests(),
 	};
 }
 
