@@ -49,6 +49,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/wrap/slide_wrap.h"
 #include "ui/wrap/vertical_layout.h"
 #include "ui/painter.h"
+#include "ui/power_saving.h"
 #include "window/window_controller.h"
 #include "window/window_session_controller.h"
 #include "base/unixtime.h"
@@ -560,7 +561,7 @@ void EmojiStatusTopBar::paint(QPainter &p) {
 				: QColor(0, 0, 0, 0)),
 			false,
 			crl::now(),
-			_paused);
+			_paused || On(PowerSaving::kEmojiStatus));
 
 		p.drawImage(_rect.toRect(), frame.image);
 		if (!_paused) {
