@@ -13,8 +13,12 @@ namespace Ui {
 
 struct InviteLinkFields {
 	QString link;
+	QString label;
 	TimeId expireDate = 0;
 	int usageLimit = 0;
+	bool requestApproval = false;
+	bool isGroup = false;
+	bool isPublic = false;
 };
 
 void EditInviteLinkBox(
@@ -24,6 +28,8 @@ void EditInviteLinkBox(
 
 void CreateInviteLinkBox(
 	not_null<Ui::GenericBox*> box,
+	bool isGroup,
+	bool isPublic,
 	Fn<void(InviteLinkFields)> done);
 
 } // namespace Ui
