@@ -318,6 +318,7 @@ Panel::Panel(
 	Fn<bool(QString)> handleLocalUri,
 	Fn<void(QString)> handleInvoice,
 	Fn<void(QByteArray)> sendData,
+	Fn<void(std::vector<QString>, QString)> switchInlineQuery,
 	Fn<void()> close,
 	QString phone,
 	MenuButtons menuButtons,
@@ -328,6 +329,7 @@ Panel::Panel(
 , _handleLocalUri(std::move(handleLocalUri))
 , _handleInvoice(std::move(handleInvoice))
 , _sendData(std::move(sendData))
+, _switchInlineQuery(std::move(switchInlineQuery))
 , _close(std::move(close))
 , _phone(phone)
 , _menuButtons(menuButtons)
@@ -625,6 +627,8 @@ bool Panel::createWebview() {
 			_close();
 		} else if (command == "web_app_data_send") {
 			sendDataMessage(arguments);
+		} else if (command == "web_app_switch_inline_query") {
+			switchInlineQueryMessage(arguments);
 		} else if (command == "web_app_setup_main_button") {
 			processMainButtonMessage(arguments);
 		} else if (command == "web_app_setup_back_button") {
@@ -702,6 +706,38 @@ void Panel::sendDataMessage(const QJsonObject &args) {
 	_sendData(data.toUtf8());
 }
 
+void Panel::switchInlineQueryMessage(const QJsonObject &args) {
+	if (args.isEmpty()) {
+		_close();
+		return;
+	}
+	const auto query = args["query"].toString();
+	if (query.isEmpty()) {
+		LOG(("BotWebView Error: Bad 'query' in switchInlineQueryMessage."));
+		_close();
+		return;
+	}
+	const auto valid = base::flat_set<QString>{
+		u"users"_q,
+		u"bots"_q,
+		u"groups"_q,
+		u"channels"_q,
+	};
+	auto types = std::vector<QString>();
+	for (const auto &value : args["chat_types"].toArray()) {
+		const auto type = value.toString();
+		if (valid.contains(type)) {
+			types.push_back(type);
+		} else {
+			LOG(("BotWebView Error: "
+				"Bad chat type in switchInlineQueryMessage: %1.").arg(type));
+			types.clear();
+			break;
+		}
+	}
+	_switchInlineQuery(types, query);
+}
+
 void Panel::openTgLink(const QJsonObject &args) {
 	if (args.isEmpty()) {
 		_close();
@@ -1107,7 +1143,7 @@ void Panel::showWebviewError(
 				"https://go.microsoft.com/fwlink/p/?LinkId=2124703"))
 			.append(parts.value(1));
 	} break;
-	case Error::NoGtkOrWebkit2Gtk:
+	case Error::NoWebKitGTK:
 		rich.append(tr::lng_payments_webview_install_webkit(tr::now));
 		break;
 	default:
@@ -1129,6 +1165,7 @@ std::unique_ptr<Panel> Show(Args &&args) {
 		std::move(args.handleLocalUri),
 		std::move(args.handleInvoice),
 		std::move(args.sendData),
+		std::move(args.switchInlineQuery),
 		std::move(args.close),
 		args.phone,
 		args.menuButtons,
