@@ -14,6 +14,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_session.h"
 #include "lang/lang_keys.h"
 #include "ui/painter.h"
+#include "ui/power_saving.h"
 #include "ui/unread_badge_paint.h"
 #include "styles/style_dialogs.h"
 
@@ -195,7 +196,7 @@ int PeerBadge::drawGetWidth(
 			.position = QPoint(
 				iconx - 2 * _emojiStatus->skip,
 				icony + _emojiStatus->skip),
-			.paused = descriptor.paused,
+			.paused = descriptor.paused || On(PowerSaving::kEmojiStatus),
 		});
 		return iconw - 4 * _emojiStatus->skip;
 	}
