@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "window/main_window.h"
 
+#include "api/api_updates.h"
 #include "storage/localstorage.h"
 #include "platform/platform_specific.h"
 #include "ui/platform/ui_platform_window.h"
@@ -49,6 +50,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include <QtGui/QScreen>
 #include <QtGui/QDrag>
 
+#include <kurlmimedata.h>
+
 namespace Window {
 namespace {
 
@@ -64,6 +67,11 @@ using Core::WindowPosition;
 	return { skipx, skipy };
 }
 
+[[nodiscard]] QImage &OverridenIcon() {
+	static auto result = QImage();
+	return result;
+}
+
 } // namespace
 
 const QImage &Logo() {
@@ -123,12 +131,19 @@ void ConvertIconToBlack(QImage &image) {
 	}
 }
 
+void OverrideApplicationIcon(QImage image) {
+	OverridenIcon() = std::move(image);
+}
+
 QIcon CreateOfficialIcon(Main::Session *session) {
 	const auto support = (session && session->supportMode());
 	if (!support) {
 		return QIcon();
 	}
-	auto image = Logo();
+	auto overriden = OverridenIcon();
+	auto image = overriden.isNull()
+		? Platform::DefaultApplicationIcon()
+		: overriden;
 	ConvertIconToBlack(image);
 	return QIcon(Ui::PixmapFromImage(std::move(image)));
 }
@@ -234,7 +249,10 @@ QImage GenerateCounterLayer(CounterLayerArgs &&args) {
 		}
 	}();
 
-	auto result = QImage(d.size, d.size, QImage::Format_ARGB32);
+	auto result = QImage(
+		QSize(d.size, d.size) * args.devicePixelRatio,
+		QImage::Format_ARGB32);
+	result.setDevicePixelRatio(args.devicePixelRatio);
 	result.fill(Qt::transparent);
 
 	auto p = QPainter(&result);
@@ -498,9 +516,13 @@ void MainWindow::handleStateChanged(Qt::WindowState state) {
 }
 
 void MainWindow::handleActiveChanged() {
+	checkActivation();
 	if (isActiveWindow()) {
 		Core::App().windowActivated(&controller());
 	}
+	if (const auto controller = sessionController()) {
+		controller->session().updates().updateOnline();
+	}
 }
 
 void MainWindow::handleVisibleChanged(bool visible) {
