@@ -40,6 +40,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include <QtGui/QWindow>
 
+#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+#include <giomm.h>
+#endif // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+
 namespace Window {
 namespace Notifications {
 namespace {
@@ -85,7 +89,13 @@ base::options::toggle OptionGNotification({
 	.name = "GNotification",
 	.description = "Force enable GLib's GNotification."
 		" When disabled, autodetect is used.",
-	.scope = base::options::linux,
+	.scope = [] {
+#ifndef DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+		return bool(Gio::Application::get_default());
+#else // !DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+		return false;
+#endif // DESKTOP_APP_DISABLE_DBUS_INTEGRATION
+	},
 	.restartRequired = true,
 });
 
@@ -564,22 +574,28 @@ void System::showNext() {
 	}
 	const auto &settings = Core::App().settings();
 	if (alertThread) {
-		if (settings.flashBounceNotify() && !_manager->skipFlashBounce()) {
+		if (settings.flashBounceNotify()) {
 			const auto peer = alertThread->peer();
 			if (const auto window = Core::App().windowFor(peer)) {
-				if (const auto handle = window->widget()->windowHandle()) {
-					handle->alert(kSystemAlertDuration);
-					// (handle, SLOT(_q_clearAlert())); in the future.
+				if (const auto controller = window->sessionController()) {
+					_manager->maybeFlashBounce(crl::guard(controller, [=] {
+						if (const auto handle = window->widget()->windowHandle()) {
+							handle->alert(kSystemAlertDuration);
+							// (handle, SLOT(_q_clearAlert())); in the future.
+						}
+					}));
 				}
 			}
 		}
-		if (settings.soundNotify() && !_manager->skipAudio()) {
-			const auto track = lookupSound(
-				&alertThread->owner(),
-				alertThread->owner().notifySettings().sound(alertThread).id);
-			track->playOnce();
-			Media::Player::mixer()->suppressAll(track->getLengthMs());
-			Media::Player::mixer()->faderOnTimer();
+		if (settings.soundNotify()) {
+			const auto owner = &alertThread->owner();
+			const auto id = owner->notifySettings().sound(alertThread).id;
+			_manager->maybePlaySound(crl::guard(&owner->session(), [=] {
+				const auto track = lookupSound(owner, id);
+				track->playOnce();
+				Media::Player::mixer()->suppressAll(track->getLengthMs());
+				Media::Player::mixer()->scheduleFaderCallback();
+			}));
 		}
 	}
 
