@@ -12,6 +12,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/style/style_core_palette.h"
 
 class QFileSystemWatcher;
+struct SendMediaReady;
 
 namespace style {
 struct colorizer;
@@ -296,6 +297,10 @@ private:
 
 };
 
+[[nodiscard]] SendMediaReady PrepareWallPaper(
+	MTP::DcId dcId,
+	const QImage &image);
+
 [[nodiscard]] ChatBackground *Background();
 
 bool ReadPaletteValues(
