@@ -18,6 +18,8 @@ namespace Theme {
 namespace {
 
 constexpr auto kMaxAccentColors = 3;
+constexpr auto kDayBaseFile = ":/gui/day-custom-base.tdesktop-theme"_cs;
+constexpr auto kNightBaseFile = ":/gui/night-custom-base.tdesktop-theme"_cs;
 
 const auto kColorizeIgnoredKeys = base::flat_set<QLatin1String>{ {
 	qstr("boxTextFgGood"),
@@ -47,6 +49,14 @@ const auto kColorizeIgnoredKeys = base::flat_set<QLatin1String>{ {
 	qstr("historyPeer8NameFg"),
 	qstr("historyPeer8NameFgSelected"),
 	qstr("historyPeer8UserpicBg"),
+	qstr("historyPeer1UserpicBg2"),
+	qstr("historyPeer2UserpicBg2"),
+	qstr("historyPeer3UserpicBg2"),
+	qstr("historyPeer4UserpicBg2"),
+	qstr("historyPeer5UserpicBg2"),
+	qstr("historyPeer6UserpicBg2"),
+	qstr("historyPeer7UserpicBg2"),
+	qstr("historyPeer8UserpicBg2"),
 	qstr("msgFile1Bg"),
 	qstr("msgFile1BgDark"),
 	qstr("msgFile1BgOver"),
@@ -303,6 +313,40 @@ std::vector<QColor> DefaultAccentColors(EmbeddedType type) {
 	Unexpected("Type in Window::Theme::AccentColors.");
 }
 
+Fn<void(style::palette&)> PreparePaletteCallback(
+		bool dark,
+		std::optional<QColor> accent) {
+	return [=](style::palette &palette) {
+		using namespace Theme;
+		const auto &embedded = EmbeddedThemes();
+		const auto i = ranges::find(
+			embedded,
+			dark ? EmbeddedType::Night : EmbeddedType::Default,
+			&EmbeddedScheme::type);
+		Assert(i != end(embedded));
+		const auto colorizer = accent
+			? ColorizerFrom(*i, *accent)
+			: style::colorizer();
+
+		auto instance = Instance();
+		const auto loaded = LoadFromFile(
+			(dark ? kNightBaseFile : kDayBaseFile).utf16(),
+			&instance,
+			nullptr,
+			nullptr,
+			colorizer);
+		Assert(loaded);
+		palette.finalize();
+		palette = instance.palette;
+	};
+}
+
+Fn<void(style::palette&)> PrepareCurrentPaletteCallback() {
+	return [=, data = style::main_palette::save()](style::palette &palette) {
+		palette.load(data);
+	};
+}
+
 QByteArray AccentColors::serialize() const {
 	auto result = QByteArray();
 	if (_data.empty()) {
