@@ -34,6 +34,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "styles/style_widgets.h"
 #include "styles/style_window.h"
+#include "styles/style_layers.h" // attentionBoxButton
 #include "styles/style_menu_icons.h"
 
 namespace Window {
@@ -235,11 +236,6 @@ void FiltersMenu::refresh() {
 	// After the filters are refreshed, the scroll is reset,
 	// so we have to restore it.
 	_scroll.scrollToY(oldTop);
-	const auto i = _filters.find(_activeFilterId);
-	const auto button = ((i != end(_filters)) ? i->second.get() : nullptr);
-	if (button) {
-		scrollToButton(button);
-	}
 }
 
 void FiltersMenu::setupList() {
@@ -411,23 +407,98 @@ void FiltersMenu::showEditBox(FilterId id) {
 }
 
 void FiltersMenu::showRemoveBox(FilterId id) {
-	_session->window().show(Ui::MakeConfirmBox({
-		.text = tr::lng_filters_remove_sure(),
-		.confirmed = [=](Fn<void()> &&close) { close(); remove(id); },
-		.confirmText = tr::lng_filters_remove_yes(),
-	}));
+	const auto session = &_session->session();
+	const auto &list = session->data().chatsFilters().list();
+	const auto i = ranges::find(list, id, &Data::ChatFilter::id);
+	const auto filter = (i != end(list)) ? *i : Data::ChatFilter();
+	const auto has = filter.hasMyLinks();
+	const auto confirm = [=](Fn<void()> action, bool onlyWhenHas = false) {
+		if (!has && onlyWhenHas) {
+			action();
+			return;
+		}
+		_session->window().show(Ui::MakeConfirmBox({
+			.text = (has
+				? tr::lng_filters_delete_sure()
+				: tr::lng_filters_remove_sure()),
+			.confirmed = [=](Fn<void()> &&close) { close(); action(); },
+			.confirmText = (has
+				? tr::lng_box_delete()
+				: tr::lng_filters_remove_yes()),
+			.confirmStyle = &st::attentionBoxButton,
+		}));
+	};
+	const auto simple = [=] {
+		confirm([=] { remove(id); });
+	};
+	const auto suggestRemoving = Api::ExtractSuggestRemoving(filter);
+	if (suggestRemoving.empty()) {
+		simple();
+		return;
+	} else if (_removingRequestId) {
+		if (_removingId == id) {
+			return;
+		}
+		session->api().request(_removingRequestId).cancel();
+	}
+	_removingId = id;
+	_removingRequestId = session->api().request(
+		MTPchatlists_GetLeaveChatlistSuggestions(
+			MTP_inputChatlistDialogFilter(
+				MTP_int(id)))
+	).done(crl::guard(&_outer, [=](const MTPVector<MTPPeer> &result) {
+		_removingRequestId = 0;
+		const auto suggestRemovePeers = ranges::views::all(
+			result.v
+		) | ranges::views::transform([=](const MTPPeer &peer) {
+			return session->data().peer(peerFromMTP(peer));
+		}) | ranges::to_vector;
+		const auto chosen = crl::guard(&_outer, [=](
+				std::vector<not_null<PeerData*>> peers) {
+			remove(id, std::move(peers));
+		});
+		confirm(crl::guard(&_outer, [=] {
+			Api::ProcessFilterRemove(
+				_session,
+				filter.title(),
+				filter.iconEmoji(),
+				suggestRemoving,
+				suggestRemovePeers,
+				chosen);
+		}), true);
+	})).fail(crl::guard(&_outer, [=] {
+		_removingRequestId = 0;
+		simple();
+	})).send();
 }
 
-void FiltersMenu::remove(FilterId id) {
-	_session->session().data().chatsFilters().apply(MTP_updateDialogFilter(
+void FiltersMenu::remove(
+		FilterId id,
+		std::vector<not_null<PeerData*>> leave) {
+	const auto session = &_session->session();
+	const auto api = &session->api();
+	session->data().chatsFilters().apply(MTP_updateDialogFilter(
 		MTP_flags(MTPDupdateDialogFilter::Flag(0)),
 		MTP_int(id),
 		MTPDialogFilter()));
-	_session->session().api().request(MTPmessages_UpdateDialogFilter(
-		MTP_flags(MTPmessages_UpdateDialogFilter::Flag(0)),
-		MTP_int(id),
-		MTPDialogFilter()
-	)).send();
+	if (leave.empty()) {
+		api->request(MTPmessages_UpdateDialogFilter(
+			MTP_flags(MTPmessages_UpdateDialogFilter::Flag(0)),
+			MTP_int(id),
+			MTPDialogFilter()
+		)).send();
+	} else {
+		api->request(MTPchatlists_LeaveChatlist(
+			MTP_inputChatlistDialogFilter(MTP_int(id)),
+			MTP_vector<MTPInputPeer>(ranges::views::all(
+				leave
+			) | ranges::views::transform([](not_null<PeerData*> peer) {
+				return MTPInputPeer(peer->input);
+			}) | ranges::to<QVector>())
+		)).done([=](const MTPUpdates &result) {
+			api->applyUpdates(result);
+		}).send();
+	}
 }
 
 void FiltersMenu::applyReorder(
