@@ -71,6 +71,7 @@ enum class CloudThemeType;
 class Thread;
 class Forum;
 class ForumTopic;
+class WallPaper;
 } // namespace Data
 
 namespace HistoryView::Reactions {
@@ -97,6 +98,7 @@ inline constexpr bool is_flag_type(GifPauseReason) { return true; };
 
 enum class ResolveType {
 	Default,
+	BotApp,
 	BotStart,
 	AddToGroup,
 	AddToChannel,
@@ -107,6 +109,7 @@ enum class ResolveType {
 struct PeerThemeOverride {
 	PeerData *peer = nullptr;
 	std::shared_ptr<Ui::ChatTheme> theme;
+	EmojiPtr emoji = nullptr;
 };
 bool operator==(const PeerThemeOverride &a, const PeerThemeOverride &b);
 bool operator!=(const PeerThemeOverride &a, const PeerThemeOverride &b);
@@ -171,6 +174,7 @@ struct SectionShow {
 	anim::activation activation = anim::activation::normal;
 	bool thirdColumn = false;
 	bool childColumn = false;
+	bool reapplyLocalDraft = false;
 	Origin origin;
 
 };
@@ -207,6 +211,8 @@ public:
 		QString startToken;
 		ChatAdminRights startAdminRights;
 		bool startAutoSubmit = false;
+		QString botAppName;
+		bool botAppForceConfirmation = false;
 		QString attachBotUsername;
 		std::optional<QString> attachBotToggleCommand;
 		InlineBots::PeerTypes attachBotChooseTypes;
@@ -373,6 +379,17 @@ public:
 	rpl::producer<Dialogs::Key> activeChatValue() const;
 	bool jumpToChatListEntry(Dialogs::RowDescriptor row);
 
+	void setCurrentDialogsEntryState(Dialogs::EntryState state);
+	[[nodiscard]] Dialogs::EntryState currentDialogsEntryState() const;
+	bool switchInlineQuery(
+		Dialogs::EntryState to,
+		not_null<UserData*> bot,
+		const QString &query);
+	bool switchInlineQuery(
+		not_null<Data::Thread*> thread,
+		not_null<UserData*> bot,
+		const QString &query);
+
 	[[nodiscard]] Dialogs::RowDescriptor resolveChatNext(
 		Dialogs::RowDescriptor from = {}) const;
 	[[nodiscard]] Dialogs::RowDescriptor resolveChatPrevious(
@@ -481,7 +498,10 @@ public:
 		not_null<PeerData*> peer,
 		MsgId msgId = ShowAtUnreadMsgId);
 
-	void toggleChooseChatTheme(not_null<PeerData*> peer);
+	void toggleChooseChatTheme(
+		not_null<PeerData*> peer,
+		std::optional<bool> show = std::nullopt);
+	void finishChatThemeEdit(not_null<PeerData*> peer);
 
 	[[nodiscard]] bool dialogsListFocused() const {
 		return _dialogsListFocused.current();
@@ -523,8 +543,13 @@ public:
 	}
 	[[nodiscard]] auto cachedChatThemeValue(
 		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
 		Data::CloudThemeType type)
 	-> rpl::producer<std::shared_ptr<Ui::ChatTheme>>;
+	[[nodiscard]] bool chatThemeAlreadyCached(
+		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
+		Data::CloudThemeType type);
 	void setChatStyleTheme(const std::shared_ptr<Ui::ChatTheme> &theme);
 	void clearCachedChatThemes();
 	void pushLastUsedChatTheme(const std::shared_ptr<Ui::ChatTheme> &theme);
@@ -532,7 +557,8 @@ public:
 
 	void overridePeerTheme(
 		not_null<PeerData*> peer,
-		std::shared_ptr<Ui::ChatTheme> theme);
+		std::shared_ptr<Ui::ChatTheme> theme,
+		EmojiPtr emoji);
 	void clearPeerThemeOverride(not_null<PeerData*> peer);
 	[[nodiscard]] auto peerThemeOverrideValue() const
 		-> rpl::producer<PeerThemeOverride> {
@@ -571,6 +597,7 @@ public:
 	}
 
 private:
+	struct CachedThemeKey;
 	struct CachedTheme;
 
 	void init();
@@ -601,7 +628,9 @@ private:
 
 	void pushDefaultChatBackground();
 	void cacheChatTheme(
+		CachedThemeKey key,
 		const Data::CloudTheme &data,
+		const Data::WallPaper &paper,
 		Data::CloudThemeType type);
 	void cacheChatThemeDone(std::shared_ptr<Ui::ChatTheme> result);
 	void updateCustomThemeBackground(CachedTheme &theme);
@@ -635,6 +664,8 @@ private:
 	int _chatEntryHistoryPosition = -1;
 	bool _filtersActivated = false;
 
+	Dialogs::EntryState _currentDialogsEntryState;
+
 	base::Timer _invitePeekTimer;
 
 	rpl::variable<FilterId> _activeChatsFilter;
@@ -651,7 +682,7 @@ private:
 	rpl::event_stream<> _filtersMenuChanged;
 
 	std::shared_ptr<Ui::ChatTheme> _defaultChatTheme;
-	base::flat_map<Ui::ChatThemeKey, CachedTheme> _customChatThemes;
+	base::flat_map<CachedThemeKey, CachedTheme> _customChatThemes;
 	rpl::event_stream<std::shared_ptr<Ui::ChatTheme>> _cachedThemesStream;
 	const std::unique_ptr<Ui::ChatStyle> _chatStyle;
 	std::weak_ptr<Ui::ChatTheme> _chatStyleTheme;
