@@ -25,7 +25,6 @@
 #include "api/call/audio_sink.h"
 #include "modules/audio_processing/audio_buffer.h"
 #include "absl/strings/match.h"
-#include "pc/channel_manager.h"
 #include "audio/audio_state.h"
 #include "modules/audio_coding/neteq/default_neteq_factory.h"
 #include "modules/audio_coding/include/audio_coding_module.h"
@@ -376,13 +375,14 @@ public:
     _videoCapture(descriptor.videoCapture) {
         webrtc::field_trial::InitFieldTrialsFromString(
             "WebRTC-DataChannel-Dcsctp/Enabled/"
+            "WebRTC-Audio-iOS-Holding/Enabled/"
         );
     }
 
     ~InstanceV2ReferenceImplInternal() {
         _currentStrongSink.reset();
 
-        _threads->getWorkerThread()->Invoke<void>(RTC_FROM_HERE, [&]() {
+        _threads->getWorkerThread()->BlockingCall([&]() {
             _audioDeviceModule = nullptr;
         });
 
@@ -444,7 +444,7 @@ public:
 
         _signalingConnection->start();
 
-        _threads->getWorkerThread()->Invoke<void>(RTC_FROM_HERE, [&]() {
+        _threads->getWorkerThread()->BlockingCall([&]() {
             _audioDeviceModule = createAudioDeviceModule();
         });
 
@@ -458,6 +458,11 @@ public:
 
         cricket::MediaEngineDependencies mediaDeps;
         mediaDeps.adm = _audioDeviceModule;
+        
+        webrtc:: AudioProcessingBuilder builder;
+        mediaDeps.audio_processing = builder.Create();
+
+        
         mediaDeps.task_queue_factory = peerConnectionFactoryDependencies.task_queue_factory.get();
         mediaDeps.audio_encoder_factory = webrtc::CreateAudioEncoderFactory<webrtc::AudioEncoderOpus>();
         mediaDeps.audio_decoder_factory = webrtc::CreateAudioDecoderFactory<webrtc::AudioDecoderOpus>();
@@ -648,10 +653,8 @@ public:
             if (server.isTurn) {
                 webrtc::PeerConnectionInterface::IceServer mappedServer;
 
-                std::ostringstream uri;
-                uri << "turn:" << address.HostAsURIString() << ":" << server.port;
-
-                mappedServer.urls.push_back(uri.str());
+                mappedServer.urls.push_back(
+                    "turn:" + address.HostAsURIString() + ":" + std::to_string(server.port));
                 mappedServer.username = server.login;
                 mappedServer.password = server.password;
 
@@ -659,10 +662,8 @@ public:
             } else {
                 webrtc::PeerConnectionInterface::IceServer mappedServer;
 
-                std::ostringstream uri;
-                uri << "stun:" << address.HostAsURIString() << ":" << server.port;
-
-                mappedServer.urls.push_back(uri.str());
+                mappedServer.urls.push_back(
+                    "stun:" + address.HostAsURIString() + ":" + std::to_string(server.port));
 
                 peerConnectionConfiguration.servers.push_back(mappedServer);
             }
@@ -690,7 +691,7 @@ public:
             cricket::AudioOptions audioSourceOptions;
             rtc::scoped_refptr<webrtc::AudioSourceInterface> audioSource = _peerConnectionFactory->CreateAudioSource(audioSourceOptions);
 
-            rtc::scoped_refptr<webrtc::AudioTrackInterface> audioTrack = _peerConnectionFactory->CreateAudioTrack("0", audioSource);
+            rtc::scoped_refptr<webrtc::AudioTrackInterface> audioTrack = _peerConnectionFactory->CreateAudioTrack("0", audioSource.get());
             webrtc::RTCErrorOr<rtc::scoped_refptr<webrtc::RtpTransceiverInterface>> audioTransceiverOrError = _peerConnection->AddTransceiver(audioTrack, transceiverInit);
             if (audioTransceiverOrError.ok()) {
                 _outgoingAudioTrack = audioTrack;
@@ -732,7 +733,7 @@ public:
                         }
 
                         strong->sendPendingSignalingServiceData(cause);
-                    }, delayMs);
+                    }, webrtc::TimeDelta::Millis(delayMs));
                 }
             }
         );
@@ -815,7 +816,7 @@ public:
             strong->writeStateLogRecords();
 
             strong->beginLogTimer(1000);
-        }, delayMs);
+        }, webrtc::TimeDelta::Millis(delayMs));
     }
 
     void writeStateLogRecords() {
@@ -1307,7 +1308,7 @@ public:
             } else {
                 _videoCapture = videoCapture;
 
-                auto videoTrack = _peerConnectionFactory->CreateVideoTrack("1", videoCaptureImpl->source());
+                auto videoTrack = _peerConnectionFactory->CreateVideoTrack("1", videoCaptureImpl->source().get());
                 if (videoTrack) {
                     webrtc::RtpTransceiverInit transceiverInit;
                     transceiverInit.stream_ids = { "0" };
@@ -1415,9 +1416,11 @@ public:
     }
 
     void setAudioInputDevice(std::string id) {
+        SetAudioInputDeviceById(_audioDeviceModule.get(), id);
     }
 
     void setAudioOutputDevice(std::string id) {
+        SetAudioOutputDeviceById(_audioDeviceModule.get(), id);
     }
 
     void setIsLowBatteryLevel(bool isLowBatteryLevel) {
@@ -1448,14 +1451,10 @@ public:
         for (const auto &record : _networkStateLogRecords) {
             json11::Json::object jsonRecord;
 
-            std::ostringstream timestampString;
-
             if (baseTimestamp == 0) {
                 baseTimestamp = record.timestamp;
             }
-            timestampString << (record.timestamp - baseTimestamp);
-
-            jsonRecord.insert(std::make_pair("t", json11::Json(timestampString.str())));
+            jsonRecord.insert(std::make_pair("t", json11::Json(std::to_string(record.timestamp - baseTimestamp))));
             jsonRecord.insert(std::make_pair("c", json11::Json(record.record.isConnected ? 1 : 0)));
             if (record.record.route) {
                 jsonRecord.insert(std::make_pair("local", json11::Json(record.record.route->localDescription)));
@@ -1622,7 +1621,7 @@ InstanceV2ReferenceImpl::InstanceV2ReferenceImpl(Descriptor &&descriptor) {
     _internal.reset(new ThreadLocalObject<InstanceV2ReferenceImplInternal>(_threads->getMediaThread(), [descriptor = std::move(descriptor), threads = _threads]() mutable {
         return new InstanceV2ReferenceImplInternal(std::move(descriptor), threads);
     }));
-    _internal->perform(RTC_FROM_HERE, [](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([](InstanceV2ReferenceImplInternal *internal) {
         internal->start();
     });
 }
@@ -1632,55 +1631,55 @@ InstanceV2ReferenceImpl::~InstanceV2ReferenceImpl() {
 }
 
 void InstanceV2ReferenceImpl::receiveSignalingData(const std::vector<uint8_t> &data) {
-    _internal->perform(RTC_FROM_HERE, [data](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([data](InstanceV2ReferenceImplInternal *internal) {
         internal->receiveSignalingData(data);
     });
 }
 
 void InstanceV2ReferenceImpl::setVideoCapture(std::shared_ptr<VideoCaptureInterface> videoCapture) {
-    _internal->perform(RTC_FROM_HERE, [videoCapture](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([videoCapture](InstanceV2ReferenceImplInternal *internal) {
         internal->setVideoCapture(videoCapture);
     });
 }
 
 void InstanceV2ReferenceImpl::setRequestedVideoAspect(float aspect) {
-    _internal->perform(RTC_FROM_HERE, [aspect](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([aspect](InstanceV2ReferenceImplInternal *internal) {
         internal->setRequestedVideoAspect(aspect);
     });
 }
 
 void InstanceV2ReferenceImpl::setNetworkType(NetworkType networkType) {
-    _internal->perform(RTC_FROM_HERE, [networkType](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([networkType](InstanceV2ReferenceImplInternal *internal) {
         internal->setNetworkType(networkType);
     });
 }
 
 void InstanceV2ReferenceImpl::setMuteMicrophone(bool muteMicrophone) {
-    _internal->perform(RTC_FROM_HERE, [muteMicrophone](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([muteMicrophone](InstanceV2ReferenceImplInternal *internal) {
         internal->setMuteMicrophone(muteMicrophone);
     });
 }
 
 void InstanceV2ReferenceImpl::setIncomingVideoOutput(std::weak_ptr<rtc::VideoSinkInterface<webrtc::VideoFrame>> sink) {
-    _internal->perform(RTC_FROM_HERE, [sink](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([sink](InstanceV2ReferenceImplInternal *internal) {
         internal->setIncomingVideoOutput(sink);
     });
 }
 
 void InstanceV2ReferenceImpl::setAudioInputDevice(std::string id) {
-    _internal->perform(RTC_FROM_HERE, [id](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([id](InstanceV2ReferenceImplInternal *internal) {
         internal->setAudioInputDevice(id);
     });
 }
 
 void InstanceV2ReferenceImpl::setAudioOutputDevice(std::string id) {
-    _internal->perform(RTC_FROM_HERE, [id](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([id](InstanceV2ReferenceImplInternal *internal) {
         internal->setAudioOutputDevice(id);
     });
 }
 
 void InstanceV2ReferenceImpl::setIsLowBatteryLevel(bool isLowBatteryLevel) {
-    _internal->perform(RTC_FROM_HERE, [isLowBatteryLevel](InstanceV2ReferenceImplInternal *internal) {
+    _internal->perform([isLowBatteryLevel](InstanceV2ReferenceImplInternal *internal) {
         internal->setIsLowBatteryLevel(isLowBatteryLevel);
     });
 }
@@ -1736,7 +1735,7 @@ void InstanceV2ReferenceImpl::stop(std::function<void(FinalState)> completion) {
     if (_logSink) {
         debugLog = _logSink->result();
     }
-    _internal->perform(RTC_FROM_HERE, [completion, debugLog = std::move(debugLog)](InstanceV2ReferenceImplInternal *internal) mutable {
+    _internal->perform([completion, debugLog = std::move(debugLog)](InstanceV2ReferenceImplInternal *internal) mutable {
         internal->stop([completion, debugLog = std::move(debugLog)](FinalState finalState) mutable {
             finalState.debugLog = debugLog;
             completion(finalState);
