@@ -95,6 +95,7 @@ public:
 		return result;
 	}
 
+	virtual ShowFactory showFactory() = 0;
 	virtual QPointer<QWidget> outerContainer() = 0;
 
 };
@@ -190,6 +191,9 @@ public:
 		prepare();
 		finishPrepare();
 	}
+	[[nodiscard]] bool hasDelegate() const {
+		return _delegate != nullptr;
+	}
 	[[nodiscard]] not_null<BoxContentDelegate*> getDelegate() const {
 		return _delegate;
 	}
@@ -205,6 +209,10 @@ public:
 	void scrollByDraggingDelta(int delta);
 
 	void scrollToY(int top, int bottom = -1);
+	void sendScrollViewportEvent(not_null<QEvent*> event);
+	[[nodiscard]] rpl::producer<> scrolls() const;
+	[[nodiscard]] int scrollTop() const;
+	[[nodiscard]] int scrollHeight() const;
 
 protected:
 	virtual void prepare() = 0;
@@ -352,6 +360,7 @@ private:
 class BoxShow : public Show {
 public:
 	explicit BoxShow(not_null<Ui::BoxContent*> box);
+	BoxShow(const BoxShow &other);
 	~BoxShow();
 	void showBox(
 		object_ptr<BoxContent> content,
@@ -360,9 +369,16 @@ public:
 	[[nodiscard]] not_null<QWidget*> toastParent() const override;
 	[[nodiscard]] bool valid() const override;
 	operator bool() const override;
+
 private:
-	mutable QPointer<QWidget> _toastParent;
+	BoxShow(QPointer<BoxContent> weak, ShowPtr wrapped);
+
+	bool resolve() const;
+
 	const QPointer<Ui::BoxContent> _weak;
+	mutable std::shared_ptr<Show> _wrapped;
+	rpl::lifetime _lifetime;
+
 };
 
 } // namespace Ui
