@@ -72,16 +72,11 @@ void ShowOverAll(not_null<QWidget*> widget, bool canFocus) {
 	NSWindow *wnd = [reinterpret_cast<NSView*>(widget->winId()) window];
 	[wnd setLevel:NSPopUpMenuWindowLevel];
 	if (!canFocus) {
-		[wnd setStyleMask:NSUtilityWindowMask | NSNonactivatingPanelMask];
+		[wnd setStyleMask:NSWindowStyleMaskUtilityWindow | NSWindowStyleMaskNonactivatingPanel];
 		[wnd setCollectionBehavior:NSWindowCollectionBehaviorMoveToActiveSpace|NSWindowCollectionBehaviorStationary|NSWindowCollectionBehaviorFullScreenAuxiliary|NSWindowCollectionBehaviorIgnoresCycle];
 	}
 }
 
-void BringToBack(not_null<QWidget*> widget) {
-	NSWindow *wnd = [reinterpret_cast<NSView*>(widget->winId()) window];
-	[wnd setLevel:NSModalPanelWindowLevel];
-}
-
 void DrainMainQueue() {
 #ifndef OS_MAC_STORE
 	_dispatch_main_queue_callback_4CF(nullptr);
