@@ -259,29 +259,29 @@ int32_t AudioDeviceOpenAL::Init() {
 	if (!alcSetThreadContext) {
 		return -1;
 	}
-	alEventCallbackSOFT = (ALEVENTCALLBACKSOFT)alcGetProcAddress(
-		nullptr,
+	alEventCallbackSOFT = (ALEVENTCALLBACKSOFT)alGetProcAddress(
 		"alEventCallbackSOFT");
 
-	alGetSourcei64vSOFT = (ALGETSOURCEI64VSOFT)alcGetProcAddress(
-		nullptr,
+	alGetSourcei64vSOFT = (ALGETSOURCEI64VSOFT)alGetProcAddress(
 		"alGetSourcei64vSOFT");
 
 	alcGetInteger64vSOFT = (ALCGETINTEGER64VSOFT)alcGetProcAddress(
 		nullptr,
 		"alcGetInteger64vSOFT");
 
-#define RESOLVE_ENUM(ENUM) k##ENUM = alcGetEnumValue(nullptr, #ENUM)
-	RESOLVE_ENUM(AL_EVENT_CALLBACK_FUNCTION_SOFT);
-	RESOLVE_ENUM(AL_EVENT_CALLBACK_FUNCTION_SOFT);
-	RESOLVE_ENUM(AL_EVENT_CALLBACK_USER_PARAM_SOFT);
-	RESOLVE_ENUM(AL_EVENT_TYPE_BUFFER_COMPLETED_SOFT);
-	RESOLVE_ENUM(AL_EVENT_TYPE_SOURCE_STATE_CHANGED_SOFT);
-	RESOLVE_ENUM(AL_EVENT_TYPE_DISCONNECTED_SOFT);
-	RESOLVE_ENUM(AL_SAMPLE_OFFSET_CLOCK_SOFT);
-	RESOLVE_ENUM(AL_SAMPLE_OFFSET_CLOCK_EXACT_SOFT);
-	RESOLVE_ENUM(ALC_DEVICE_LATENCY_SOFT);
-#undef RESOLVE_ENUM
+#define RESOLVE_AL_ENUM(ENUM) k##ENUM = alGetEnumValue(#ENUM)
+#define RESOLVE_ALC_ENUM(ENUM) k##ENUM = alcGetEnumValue(nullptr, #ENUM)
+	RESOLVE_AL_ENUM(AL_EVENT_CALLBACK_FUNCTION_SOFT);
+	RESOLVE_AL_ENUM(AL_EVENT_CALLBACK_FUNCTION_SOFT);
+	RESOLVE_AL_ENUM(AL_EVENT_CALLBACK_USER_PARAM_SOFT);
+	RESOLVE_AL_ENUM(AL_EVENT_TYPE_BUFFER_COMPLETED_SOFT);
+	RESOLVE_AL_ENUM(AL_EVENT_TYPE_SOURCE_STATE_CHANGED_SOFT);
+	RESOLVE_AL_ENUM(AL_EVENT_TYPE_DISCONNECTED_SOFT);
+	RESOLVE_AL_ENUM(AL_SAMPLE_OFFSET_CLOCK_SOFT);
+	RESOLVE_AL_ENUM(AL_SAMPLE_OFFSET_CLOCK_EXACT_SOFT);
+	RESOLVE_ALC_ENUM(ALC_DEVICE_LATENCY_SOFT);
+#undef RESOLVE_ALC_ENUM
+#undef RESOLVE_AL_ENUM
 
 	_initialized = true;
 	return 0;
@@ -977,7 +977,7 @@ void AudioDeviceOpenAL::startPlayingOnThread() {
 				alSourcei(
 					source,
 					alGetEnumValue("AL_DIRECT_CHANNELS_SOFT"),
-					alcGetEnumValue(nullptr, "AL_REMIX_UNMATCHED_SOFT"));
+					alGetEnumValue("AL_REMIX_UNMATCHED_SOFT"));
 			}
 			_data->source = source;
 			alGenBuffers(_data->buffers.size(), _data->buffers.data());
